import {ITicket} from "@/pages/ts/tickets/interfaces";
import {useI18n} from "vue-i18n";
import {useRouter} from "vue-router";
import {useTicketStore} from "@/stores/ts/ticket";
import {Editor} from "tinymce";
import {reactive, watch} from "vue";

const useTicketSmartLinks = () => {
  const {t} = useI18n()
  const router = useRouter()
  const {getOneRow} = useTicketStore()

  const ticketSmartLinksData = reactive({
    editor: null as Editor | null,
    editorValue: '',
    editorValueWithSmartLinks: '',
  })

  const identifierAttributeValue = 'smart'
  const identifierAttributeName = 'data-link'
  const identifierTicketAttributeName = 'data-ticket'
  const mainClassName = 'smart-link'

  function createSmartLink(ticket: ITicket) {
    const $link = createLink(ticket)
    const $status = createStatus(ticket)
    const $container = createContainer(ticket)

    $link.append($status)
    $container.append($link)

    return $container
  }

  function isSmartLink(url: URL) {
    return /\/ts\/tickets\/\d+/.test(url.pathname)
  }

  async function pasteSmartLink(url: URL) {
    const ticketId = url.pathname.match(/\d+/)?.[0]

    if (ticketId) {
      const {getOneRow} = useTicketStore()

      try {
        ticketSmartLinksData.editor?.execCommand(
          'mceInsertContent',
          true,
          createSmartLink(await getOneRow(ticketId)).outerHTML
        )
      } catch {
      }
    }
  }

  function createLink(ticket: ITicket) {
    const link = router.resolve({
      name: 'ts.tickets.show',
      params: {
        id: ticket.id
      }
    })

    const $link = document.createElement('a')

    $link.setAttribute('href', link.fullPath)
    $link.setAttribute('target', '_blank')
    $link.innerText = t('ts.Ticket') + ` #${ticket.id} `

    return $link
  }

  function createStatus(ticket: ITicket) {
    const $status = document.createElement('span')
    const ticketStatus = ticket.status ? t("ts.tickets.statuses." + ticket.status) : ''

    $status.classList.add(`ts-ticket-bg-${ticket.status_color}`)
    $status.classList.add(`ts-ticket-color-${ticket.status_color}`)
    $status.innerText = ticketStatus

    return $status
  }

  function createContainer(ticket: ITicket) {
    const $container = document.createElement('span')

    $container.setAttribute('contentEditable', 'false')
    $container.setAttribute(identifierTicketAttributeName, ticket.id.toString())
    $container.setAttribute(identifierAttributeName, identifierAttributeValue)

    $container.classList.add(mainClassName)

    return $container
  }

  function parseToHtml(htmlableString: string) {
    const $dummy = document.createElement('div')

    $dummy.innerHTML = htmlableString

    return $dummy
  }

  function isSmartLinksExists(htmlableString: string) {
    if (!htmlableString) {
      return false
    }

    return !!parseToHtml(htmlableString).querySelector(`[${identifierAttributeName}="${identifierAttributeValue}"]`)
  }

  async function getDomWithLinks() {
    const $html = parseToHtml(ticketSmartLinksData.editorValue)
    const $elements = Array.from($html.querySelectorAll(`[${identifierAttributeName}="${identifierAttributeValue}"]`))

    for (const $element of $elements) {
      const ticketId = ($element as HTMLElement).dataset.ticket

      if (ticketId) {
        const ticket = await getOneRow(ticketId)

        const $link = createLink(ticket)
        const $status = createStatus(ticket)

        $link.append($status)
        $element.append($link)
      }
    }

    return $html
  }

  watch(() => ticketSmartLinksData.editor, async (editor) => {
    if (editor) {
      if (isSmartLinksExists(ticketSmartLinksData.editorValue)) {
        const htmlWithLinks = await getDomWithLinks()
        ticketSmartLinksData.editorValueWithSmartLinks = htmlWithLinks.outerHTML
      }
    }
  }, {
    immediate: true
  })

  return {
    ticketSmartLinksData,
    isSmartLink,
    pasteSmartLink,
    createSmartLink,
    parseToHtml,
  }
}

export default useTicketSmartLinks