import {reactive, watch} from "vue";
import {ISelect} from "@/utils/interfaces";
import {Editor} from "tinymce";
import {useDarkModeStore} from "@/stores/dark-mode";
import colors from "tailwindcss/colors";
import {useI18n} from "vue-i18n";
import {IEditorEmitable} from "@/base-components/TinyMCE/interfaces";

export const CONTAINER_ID_NAME = 'user-menu'
export const CONTAINER_ID = '#' + CONTAINER_ID_NAME

export const EDITOR_CONTAINER_CLASS_NAME = 'user'
const EDITOR_CONTAINER_CLASS_NAME_DARK = 'user-dark'

const useUserMenu = () => {
  const {t} = useI18n()
  const menuData = reactive({
    search: '',
    isActivated: false,
    editor: null as Editor | null,
    users: null as ISelect[] | null,
    filterUsers: null as ISelect[] | null,
    bindUsers: [] as IEditorEmitable[],
  })

  function stringToColor(string: string): string {
    let hash = 0;
    let i;

    for (i = 0; i < string.length; i += 1) {
      hash = string.charCodeAt(i) + ((hash << 5) - hash);
    }

    let color = '#';

    for (i = 0; i < 3; i += 1) {
      const value = (hash >> (i * 8)) & 0xff;
      color += `00${value.toString(16)}`.slice(-2);
    }

    return color;
  }

  function createMenuItems(container: HTMLElement, items: ISelect[]) {
    const isDarkMode = useDarkModeStore().darkModeValue

    container.innerHTML = ""

    if (items.length === 0) {
      const $empty = document.createElement('div')

      $empty.textContent = t('Users not found')

      $empty.classList.add(
        'flex',
        'items-center',
        'justify-center',
        'p-6',
        'h-[48px]',
        'shadow',
        'focus-visible:outline-none'
      )

      $empty.setAttribute('tabindex', '-1')

      container.append($empty)

      $empty.focus()
      $empty.addEventListener('keydown', onKeyPress);

      return
    }

    items.slice(0, 6).forEach((user: ISelect) => {
      const $item = document.createElement('div')
      const $avatar = document.createElement('div')
      const $fullName = document.createElement('div')

      $avatar.classList.add('icon-circle-user')
      $avatar.style.color = stringToColor(user.text.toString())
      $avatar.style.fontSize = '30px'

      $fullName.innerHTML = highlight(menuData.search.replace('@', ''), user.text)

      $item.classList.add(
        'flex',
        'items-center',
        'justify-start',
        'gap-3',
        'px-3',
        'py-2',
        'min-h-[48px]',
        'cursor-pointer',
        'border-l-transparent',
        'border-l-solid',
        'border-l-[2px]',
        'focus-visible:outline-none',
        'hover:border-l-primary',
        'focus:border-l-primary',
      );

      $item.setAttribute('tabindex', '-1')
      $item.dataset.id = user.value.toString()

      isDarkMode
        ? $item.classList.add('hover:bg-black/30', 'focus:bg-black/30')
        : $item.classList.add('hover:bg-black/10', 'focus:bg-black/10')

      $item.append($avatar);
      $item.append($fullName);

      container.append($item);

      $item.addEventListener('click', (e) => insertUserIntoContext(e.currentTarget as HTMLElement));
      $item.addEventListener('keydown', onKeyPress);
    })
  }

  function openMenu() {
    throwEditorError()

    menuData.isActivated = true;
    menuData.search += '@'

    const boundingClientRect = menuData.editor?.selection.getBoundingClientRect() as DOMRect
    const $searchUserContainer = document.createElement('span')

    removeTrigger();

    $searchUserContainer.innerText = '@'
    $searchUserContainer.setAttribute('id', 'search-user-container')

    menuData.editor?.insertContent($searchUserContainer.outerHTML)

    const container = menuData.editor?.getContainer() as HTMLElement
    const containerRect = container.getClientRects()[0] as DOMRect
    const headerRect = container.querySelector('.tox-editor-header')?.getClientRects()[0] as DOMRect

    const x = containerRect.left + boundingClientRect.left
    const y = containerRect.top + boundingClientRect.top + window.scrollY + headerRect.height + 20

    document.querySelector(CONTAINER_ID)?.remove()

    const isDarkMode = useDarkModeStore().darkModeValue
    const $wrapper = document.createElement('div');

    $wrapper.setAttribute('id', CONTAINER_ID_NAME)
    $wrapper.style.backgroundColor = isDarkMode ? colors.slate["800"] : 'white'

    $wrapper.classList.add(
      'absolute',
      'w-[250px]',
      'z-[99]',
      'shadow',
    );

    $wrapper.style.top = y + 'px'
    $wrapper.style.left = x + 'px'
    $wrapper.setAttribute('tabindex', '-1')

    createMenuItems($wrapper, menuData.users ?? [])

    document.querySelector('body')?.prepend($wrapper);
    ($wrapper.firstChild as HTMLElement).focus();
    window.addEventListener("keydown", preventBrowserScroll);
    window.addEventListener('click', (e) => closeMenuWatcher(e, $wrapper))
  }

  function closeMenu() {
    menuData.isActivated = false;
    menuData.search = '';

    const $menu = document.querySelector(CONTAINER_ID)

    if ($menu) {
      $menu.remove();
      window.removeEventListener("keydown", preventBrowserScroll);
    }
  }

  function refreshMenu(items: ISelect[]) {
    createMenuItems(
      document.querySelector(CONTAINER_ID) as HTMLElement,
      items,
    );
    (document.querySelector(CONTAINER_ID)?.firstChild as HTMLElement)?.focus()
  }

  function insertUserIntoContext(target: HTMLElement) {
    throwEditorError()

    const $container = document.createElement('span')

    $container.contentEditable = 'false'
    $container.classList.add(EDITOR_CONTAINER_CLASS_NAME)

    if (useDarkModeStore().darkModeValue) {
      $container.classList.add(EDITOR_CONTAINER_CLASS_NAME_DARK)
    }

    $container.innerText = target?.lastChild?.textContent ?? ''
    $container.innerHTML = '@' + $container.innerText + '&nbsp;'
    $container.dataset.id = target.dataset.id


    const node = menuData.editor?.selection.getRng()?.commonAncestorContainer as HTMLElement

    if (node && menuData.search.replace('@', '').length) {
      node.remove()
    }

    closeMenu()

    try {
      removeTrigger()
    }
    catch{}

    menuData.editor?.insertContent($container.outerHTML)
  }

  function onKeyPress(e: KeyboardEvent) {
    if (menuData.filterUsers?.length) {
      if (e.key === 'ArrowDown') {
        const current = e.currentTarget as HTMLElement;
        const next = current.nextSibling as HTMLElement | null

        if (!next) {
          (document.querySelector(CONTAINER_ID)?.firstChild as HTMLElement).focus()

          return;
        }

        next.focus();

        return
      }
      if (e.key === 'ArrowUp') {
        const current = e.currentTarget as HTMLElement;
        const prev = current.previousSibling as HTMLElement | null

        if (!prev) {
          (document.querySelector(CONTAINER_ID)?.lastChild as HTMLElement).focus()

          return;
        }

        prev.focus();

        return
      }
      if (e.key === 'Enter') {
        if (menuData.filterUsers?.length === 0) {
          closeMenu()

          return;
        }

        insertUserIntoContext(e.currentTarget as HTMLElement)

        return;
      }
    }

    if (e.key === 'Backspace') {
      if (menuData.search === '') {
        closeMenu()
      }

      menuData.editor?.focus()

      return;
    }

    if (e.key === 'Escape') {
      closeMenu()

      return;
    }

    if (/Digit\d+/.test(e.code)
      || /Key[A-Z]/.test(e.code)
      || e.code === 'Space'
      || /[А-я]/.test(e.key)
    ) {
      throwEditorError()

      menuData.editor?.insertContent(e.key);
      menuData.search += e.key;
    }
  }

  function isTriggered(symbol: string | null, content: string) {
    if (symbol !== '@') {
      return false
    }

    return /\s@$/gm.test(content) || /^@$/gm.test(content)
  }

  function menuModeHandler() {
    const isDarkMode = useDarkModeStore().darkModeValue

    const $wrapperInDom = document.querySelector(CONTAINER_ID) as HTMLElement

    if ($wrapperInDom) {
      $wrapperInDom.style.backgroundColor = isDarkMode ? colors.slate["800"] : 'white'
      $wrapperInDom.querySelectorAll('li')?.forEach(item => {
        isDarkMode
          ? item.classList.add('hover:bg-black/20')
          : item.classList.add('hover:bg-black/10')
      })
    }

    const body = menuData.editor?.getBody() as HTMLElement

    Array.from(body.getElementsByClassName(EDITOR_CONTAINER_CLASS_NAME)).forEach(el => {
      isDarkMode
        ? el.classList.add(EDITOR_CONTAINER_CLASS_NAME_DARK)
        : el.classList.remove(EDITOR_CONTAINER_CLASS_NAME_DARK)
    })
  }

  function throwEditorError() {
    if (!menuData.editor) {
      throw new Error('Editor not binded')
    }
  }

  function removeTrigger() {
    const editorRange = menuData.editor?.selection.getRng()

    if(!editorRange) {
      return
    }

    const node = editorRange.commonAncestorContainer;
    const range = document.createRange();

    range.selectNodeContents(node);
    range.setStart(node, editorRange.endOffset - 1);
    range.setEnd(node, editorRange.endOffset);
    range.deleteContents();

    menuData.editor?.focus();
  }

  function highlight (needle: string, text: string) {
    if (!needle) {
      return text
    }

    const index = text.search(new RegExp(needle, 'iu'))

    if (index >= 0) {
      text = text.substring(0, index)
        + '<mark class="bg-danger text-white">'
        + text.substring(index, index + needle.length)
        + '</mark>'
        + text.substring(index + needle.length, text.length)
    }

    return text
  }

  function preventBrowserScroll(e: KeyboardEvent) {
    if(["Space","ArrowUp","ArrowDown","ArrowLeft","ArrowRight"].indexOf(e.code) > -1) {
      e.preventDefault();
    }
  }

  function closeMenuWatcher(e: MouseEvent, currentTarget: HTMLElement) {
    const element = e.target as HTMLElement

    if (element.contains(currentTarget)) {
      return
    }

    closeMenu()
  }

  function bindUserHandler(htmlable: string) {
    menuData.bindUsers = []

    const $html = document.createElement('div')
    $html.innerHTML = htmlable

    $html.querySelectorAll('.' + EDITOR_CONTAINER_CLASS_NAME).forEach($userDom => menuData.bindUsers.push({
      id: Number(($userDom as HTMLElement).dataset.id),
      name: $userDom.textContent?.replace('@', '') as string,
    }))
  }

  watch(() => menuData.search, function(search) {
    if (!menuData.isActivated) {
      return;
    }

    if (/@.+null$/.test(search)) {
      closeMenu();
      return;
    }

    if (Array.from(menuData.search.matchAll(new RegExp(/\s/g))).length > 2) {
      closeMenu();
      return;
    }

    const searchString = search.replace('@', '')

    menuData.filterUsers = menuData.users?.filter(user => {
      return searchString ? user.text.search(new RegExp(searchString, 'iu')) >= 0 : user
    }) ?? [];

    refreshMenu(menuData.filterUsers)
  })

  return {
    menuData,
    states: {
      openMenu,
      closeMenu,
      isTriggered,
    },
    menuModeHandler,
    bindUserHandler,
  }
}

export default useUserMenu