import { ref, Ref, reactive, watch, onMounted } from "vue";
import {
  useRoute,
  useRouter,
  RouteRecordName,
  LocationQueryValue,
  RouteLocationNormalizedLoaded,
} from "vue-router";
import _, {isEmpty} from "lodash";
import {getPeriodStr} from "@/utils/helper";
import dayjs from "dayjs";

/* eslint-disable @typescript-eslint/no-explicit-any */
type Filter = {
  [key: string]: any,
};

type SearchField = { text: string, value: string };
type RouteParams = { [key: string]: string };
type ExtraPeriodParam = {
  name: string,
  date_begin_name: string,
  date_end_name: string,
};

export type FilterConfig = {
  stringKeys?: Array<string>,
  arrayKeys?: Array<string>,
  searchKeys?: Array<SearchField>,
  predefined?: Filter,
  hasPeriod?: boolean,
  hasDateTimePeriod?: boolean,
  hasSingleDate?: boolean,
  customInitFilterDate?: () => string,
  routeParams?: RouteParams,
  everySearchReload?: boolean,
  extraPeriodKeys?: Array<ExtraPeriodParam>,
};

export class FilterManager {
  private config: FilterConfig;

  constructor(config: FilterConfig) {
    this.config = config;
  }

  private createFilter(route?: RouteLocationNormalizedLoaded): Filter {
    const localRoute = route || useRoute();

    const stringKeyFilter = !!this.config.stringKeys
      ? Object.assign({}, ...this.config.stringKeys?.map((key: string) => ({ [key]: localRoute.query[key] || "" })))
      : {};

    const arrayKeyFilter = !!this.config.arrayKeys
      ? Object.assign({}, ...this.config.arrayKeys?.map((key: string) => ({ [key]: localRoute.query[key] || [] })))
      : {};

    const searchKeyFilter = !!this.config.searchKeys
      ? {
        filter_field: localRoute.query.filter_field || "",
        filter_value: localRoute.query.filter_value || ""
      }
      : {};

    const extraPeriodKeyFilter = !!this.config.extraPeriodKeys
      ? Object.assign(
        {},
        ...this.config.extraPeriodKeys?.map((period: ExtraPeriodParam) => ({
          [period.date_begin_name]: localRoute.query[period.date_begin_name] || "",
          [period.date_end_name]: localRoute.query[period.date_end_name] || "",
        }))
      )
      : {};

    const predefined: Filter = {};
    if (this.config.predefined) {
      const queryKeys = Object.keys(localRoute.query) || [];
      Object.keys(this.config.predefined).forEach(key => {
        if (queryKeys.length == 0 || !queryKeys.includes(key)) {
          const predefinedValue = _.get(this.config.predefined, key) as string | Array<string>;
          if (this.config.arrayKeys?.includes(key)) {
            predefined[key] = [...predefinedValue];
          } else {
            predefined[key] = predefinedValue;
          }
        }
      })
    }

    return reactive({
      ...stringKeyFilter,
      ...arrayKeyFilter,
      ...searchKeyFilter,
      ...extraPeriodKeyFilter,
      ...predefined,
      ...(this.config.hasPeriod ? {
        date_begin: localRoute.query.date_begin || predefined.date_begin || "",
        date_end: localRoute.query.date_end || predefined.date_end || "",
      } : {}),
      ...(this.config.hasSingleDate ? {
        date_begin: localRoute.query.date_begin || predefined.date_begin || "",
      } : {})
    });
  }

  public useFilter() {
    const route = useRoute();
    const router = useRouter();
    const query = ref(route.query);

    const reloadSameUrl = this.config.everySearchReload != false ? "true" : "false";

    this.config.arrayKeys?.forEach(el => {
      if (query.value[el] && !Array.isArray(query.value[el])) {
        query.value[el] = [query.value[el] as LocationQueryValue];
      }
    });

    const filter = this.createFilter();
    const filterDate: Ref<string> = ref("");
    const startDay: Ref<number| Date |null> = ref(null);
    const endDay: Ref<number| Date |null> = ref(null);
    const filterDateExtra: Ref<{ [key: string]: string }> = ref({});

    const initFilterDate = this.config.customInitFilterDate != undefined
      ? this.config.customInitFilterDate
      : () => {
        if (query.value.date_begin || query.value.date_end) {
          return query.value.date_begin + " - " + query.value.date_end;
        }
        return "";
      };

    const initFilterSingleDate = () => {
      if (query.value.date_begin) {
        return query.value.date_begin+"";
      }
      return getPeriodStr("day").split(" - ")[0];
    };
    const initFilterStartDay = () => {
      if (query.value.date_begin) {
        const [date, time] = (query.value.date_begin+'').split(' ');
        const [day, month, year] = date.split(/[. :]/);
        const formattedDateString = `${month}/${day}/${year} ${time}`;
        return new Date(formattedDateString);
      }
      return startDay.value = new Date().setHours(0,0,0,0);
    };
    const initFilterEndDay = () => {
      if (query.value.date_begin) {
        const [date, time] = (query.value.date_end+'').split(' ');
        const [day, month, year] = date.split(/[. :]/);
        const formattedDateString = `${month}/${day}/${year} ${time}`;
        return new Date(formattedDateString);
      }
      return endDay.value = new Date().setHours(23,59,59,999);
    };
    const initFilterDateExtra = () => {
      if (!this.config.extraPeriodKeys) return;

      filterDateExtra.value = Object.assign(
        {},
        ...this.config.extraPeriodKeys?.map((period: ExtraPeriodParam) => {
          const stringRepresentation = (query.value[period.date_begin_name] && query.value[period.date_end_name])
            ? `${query.value[period.date_begin_name]} - ${query.value[period.date_end_name]}`
            : "";

          return { [period.name]: stringRepresentation };
        })
      );
    };

    const searchFields = this.config.searchKeys;

    const onFilter = () => {
      router.push({
        name: route.name as RouteRecordName,
        query: {
          ...filter,
          reloadSameUrl,
        },
        params: { ...this.config.routeParams },
        force: true,
      });
    };

    const onResetFilter = () => {
      this.config.stringKeys?.forEach(function (key) {
        filter[key] = "";
      });

      this.config.arrayKeys?.forEach(function (key) {
        (filter[key] as Array<string>).splice(0, filter[key].length);
      });

      if (!!this.config.searchKeys) {
        filter.filter_field = "";
        filter.filter_value = "";
      }

      if (this.config.hasPeriod) {
        filterDate.value = initFilterDate();
      }
      if (this.config.hasSingleDate) {
        filterDate.value = initFilterSingleDate();
      }
      if (!!this.config.extraPeriodKeys) {
        initFilterDateExtra();
      }

      if (!!this.config.predefined) {
        Object.keys(this.config.predefined).forEach(key => {
          const value = (this.config.predefined as Filter)[key];

          filter[key] = !Array.isArray(value) ? value : [...value];
        });
      }

      const target = {
        name: route.name as RouteRecordName,
        params: this.config.routeParams,
      };
      if (this.config.everySearchReload === false) {
        Object.assign(target, {
          force: true,
          query: filter,
        });
      }
      router.push(target);
    }

    if (this.config.hasPeriod) {
      watch(filterDate, (newValue) => {
        const inputDates: Array<string> = newValue.split(" - ");
        if (inputDates.length > 1) {
          [filter.date_begin, filter.date_end] = inputDates;
        } else if (this.config.customInitFilterDate != undefined) {
          if (newValue != '') {
            filterDate.value = initFilterDate()
          } else {
            filter.date_begin = "";
            filter.date_end = "";
          }
        } else {
          filter.date_begin = "";
          filter.date_end = "";
        }
      });

      onMounted(() => {
        filterDate.value = initFilterDate();
      });
    }
    if (this.config.hasSingleDate) {
      watch(filterDate, (newValue) => {
        const inputDates: string = newValue;
        if (inputDates.length > 1) {
          filter.date_begin = inputDates;
        } else {
          filter.date_begin = getPeriodStr("day").split(" - ")[0];
        }
      });

      onMounted(() => {
        filterDate.value = initFilterSingleDate();
      });
    }
    //
    if(this.config.hasDateTimePeriod)  {
      if(!isEmpty(startDay)){
        startDay.value = initFilterStartDay();
        filter.date_begin = startDay.value;
      }
      if(!isEmpty(endDay)){
        endDay.value = initFilterEndDay();
        filter.date_end = endDay.value;
      }
      watch(startDay, (newValue) => {
        if (!newValue) {
          filter.date_begin = undefined;
          return;
        }
        filter.date_begin = dayjs(new Date(newValue)).format('DD.MM.YYYY HH:mm');
      }, {immediate: true});
      watch(endDay, (newValue) => {
        if (!newValue) {
          filter.date_end = undefined;
          return;
        }
        filter.date_end = dayjs(new Date(newValue)).format('DD.MM.YYYY HH:mm');
      }, {immediate: true});
    }
    if (!!this.config.extraPeriodKeys) {
      watch(() => filterDateExtra.value, (newValue) => {
        Object.keys(filterDateExtra.value).forEach((dateValue: keyof typeof filterDateExtra.value) => {
          const inputDates: Array<string> = newValue[dateValue].split(" - ");

          const config: ExtraPeriodParam = this.config.extraPeriodKeys?.filter(
            (param: ExtraPeriodParam) => param.name == dateValue
          )[0] as ExtraPeriodParam;

          if (inputDates.length > 1) {
            [
              filter[config.date_begin_name],
              filter[config.date_end_name]
            ] = inputDates;
          } else {
            filter[config.date_begin_name] = "";
            filter[config.date_end_name] = "";
          }
        })
      }, { deep: true });

      onMounted(() => {
        initFilterDateExtra();
      });
    }

    return {
      filter,
      filterDate,
      startDay,
      endDay,
      filterDateExtra,
      initFilterDate,
      searchFields,
      onFilter,
      onResetFilter,
    };
  }
}
