import { FilterConfig, FilterManager } from "@/mixins/Filter";
import defaultDate from "@/mixins/DefaultDate";
import { useI18n } from "vue-i18n";

type FilterConfigsKey = "transactions"
  | "balance"
  | "commissions"
  | "crypto_operations"
  | "available_limits"
  | "withdrawal_history"
  | "agent_merchants"
  | "gnu_vpn_tariffs"
  | "gnu_vpn_invoices";


export default (filterType: FilterConfigsKey) => {

type FilterConfigs = Record<FilterConfigsKey, FilterConfig>;

const defaultFilterDate = defaultDate();
const { t } = useI18n();

const filterConfigs: FilterConfigs = {
  transactions: {
    searchKeys: [
      {
        text: t('Invoice ID'),
        value: 'id'
      },
    ],
    stringKeys: [ "from_date", "to_date", "search", "searchField" ],
    arrayKeys: [
      "status[]",
      "invoice_types[]",
      "invoice_type[]",
      "withdrawal_types[]",
      "provider[]",
      "fiats[]",
      "payment_system_ids[]",
      "merchants[]",
    ],
    predefined: {
      from_date: defaultFilterDate,
      "status[]": [ "1" ],
    },
    routeParams: {
      section: "transactions",
    },
    everySearchReload: false,
  },
  balance: {
    stringKeys: [ "from_date", "to_date" ],
    arrayKeys: [
      "invoice_types[]",
      "provider[]",
    ],
    predefined: {
      from_date: defaultFilterDate,
    },
    routeParams: {
      section: "merchant_balance",
    },
    everySearchReload: false,
  },
  commissions: {
    stringKeys: [ "type_amount" ],
    arrayKeys: [
      "fiats[]",
      "invoice_types[]",
      "provider[]",
      "api_type[]"
    ],
    predefined: {
      type_amount: "0",
      "api_type[]": ["0"]
    },
    routeParams: {
      section: "merchant_commissions",
    },
    everySearchReload: false,
  },
  crypto_operations: {
    stringKeys: [ "from_date", "to_date" ],
    arrayKeys: [
      "invoice_types[]",
      "crypto_address[]",
    ],
    routeParams: {
      section: "merchant_crypto_operations",
    },
    everySearchReload: false,
  },
  available_limits: {
    stringKeys: [ "direction" ],
    arrayKeys: [
      "payment_system_ids[]",
      "payment_system_type_ids[]",
      "fiats[]",
    ],
    predefined: {
      direction: "0",
    },
    routeParams: {
      section: "merchant_provider_limits",
    },
    everySearchReload: false,
  },
  withdrawal_history: {
    stringKeys: [ "from_date", "to_date" ],
    arrayKeys: [ "status[]", "crypto_address[]" ],
    routeParams: {
      section: "withdrawal_history",
    },
    everySearchReload: false,
  },
  agent_merchants: {
    arrayKeys: [ "status[]" ],
    everySearchReload: false,
    routeParams: {
      section: "agent_merchants",
    },
  },
  gnu_vpn_tariffs: {
    stringKeys: [
      "date_activated_at_from",
      "date_activated_at_to",
      "date_begins_at_from",
      "date_begins_at_to",
      "date_ends_at_from",
      "date_ends_at_to",
      "is_manual",
    ],
    arrayKeys: [ "tariff_name[]" ],
    extraPeriodKeys: [
      {
        name: 'date_activated_at',
        date_begin_name: 'date_activated_at_from',
        date_end_name: 'date_activated_at_to',
      },
      {
        name: 'date_begins_at',
        date_begin_name: 'date_begins_at_from',
        date_end_name: 'date_begins_at_to',
      },
      {
        name: 'date_end',
        date_begin_name: 'date_end_from',
        date_end_name: 'date_end_to',
      },
    ],
    hasPeriod: true,
    routeParams: {
      section: "tab_tariffs",
    },
    everySearchReload: false,
  },
  gnu_vpn_invoices: {
    arrayKeys: [
      "tariff_name[]",
      "provider_name[]",
      "currency[]",
    ],
    routeParams: {
      section: "tab_invoices",
    },
    everySearchReload: false,
  },
};

return (new FilterManager(filterConfigs[filterType])).useFilter();
};
