import {
  InteractionType,
  IStatus,
  ITicket,
  ITicketForList,
  ITicketForMassActions,
  ITicketInvoice,
  ITicketOtrs,
  ITicketStatus,
  TicketCustomerTypes,
} from "@/pages/ts/tickets/interfaces";
import {ITicketComment, ITicketCommentUpdateForm} from "@/pages/ts/ticket_comments/interfaces";
import {IUserShort} from "@/pages/users/interfaces";
import {ITicketType} from "@/pages/ts/ticket_types/interfaces";
import {IQueueTopic} from "@/pages/ts/queue_topics/interfaces";
import {ITicketCustomer} from "@/pages/ts/customers/interfaces";
import {useTicketCommentStore} from "@/stores/ts/ticket_comment";
import {IFile} from "@/stores/file/interfaces";
import {IQueue} from "@/pages/ts/queues/interfaces";
import {ILog} from "@/pages/ts/log/interfaces";
import {IPriority} from "@/pages/ts/priorities/interfaces";
import {QueueRolePermissionTypeEnum} from "@/pages/ts/queue_role_permissions/interfaces";
import {ITicketDynamicField} from "@/pages/ts/ticket_dynamic_fields/interfaces";
import {ICriticalityTicket} from "@/pages/criticality/interfaces";
import {ICoefficientTicket} from "@/pages/coefficients/interfaces";
import {RequestCommentableType} from "@/utils/interfaces";
import {IReminder} from "@/stores/reminder/interfaces";

class Ticket implements ITicket {
  id: number
  parent_ticket: ITicket | null
  parent_id: number | null
  type_id: number
  source: number
  source_name: string
  subject: string
  subject_with_id: string
  text: string
  executor_id: number | null
  customer_type: TicketCustomerTypes | null
  customer_id: number | null
  customer: ITicketCustomer | null
  customer_coefficient: ICoefficientTicket | null
  date_execution: string
  date_delayed_to: string
  date_period_to: string | null
  period_step: number | null
  notify_period: number | null
  created_at: string
  updated_at: string
  closed_at: string
  email: string
  creator_formed: string
  creator_user: IUserShort | null
  executor_user: IUserShort | null
  type: ITicketType
  status: IStatus
  status_color: string
  status_transitions: Array<ITicketStatus>
  queue: IQueue
  priority: IPriority
  dynamic_fields: Array<ITicketDynamicField> | null
  queue_topic: IQueueTopic
  time_elapsed: string
  time_eta: string
  time_sla: string
  date_eta: string
  files: Array<IFile> | null
  linked_tickets: Array<ITicket> | null
  children: Array<ITicket> | null
  link_comment: string | null
  history: Array<ILog> | null
  current_user_permissions: Array<QueueRolePermissionTypeEnum> | null
  ticket_otrs: ITicketOtrs | null
  ticket_invoices: ITicketInvoice[]
  user_favorite_id: number | null
  user_review_id: number | null
  criticality?: ICriticalityTicket[]
  theme: string | null
  interaction_type?: InteractionType | null
  is_private: boolean
  reminders: IReminder[]
  disallow_reopen: boolean
  visible_to_client: boolean

  static ticketCommentStore = useTicketCommentStore()

  constructor(attrs: ITicket) {
    this.id = attrs.id;
    this.parent_ticket = attrs.parent_ticket;
    this.parent_id = attrs.parent_id;
    this.type_id = attrs.type_id;
    this.source = attrs.source;
    this.source_name = attrs.source_name;
    this.subject = attrs.subject;
    this.subject_with_id = attrs.subject_with_id;
    this.text = attrs.text;
    this.executor_id = attrs.executor_id;
    this.customer_type = attrs.customer_type;
    this.customer_id = attrs.customer_id;
    this.customer = attrs.customer;
    this.customer_coefficient = attrs.customer_coefficient;
    this.date_execution = attrs.date_execution;
    this.date_period_to = attrs.date_period_to;
    this.period_step = attrs.period_step;
    this.notify_period = attrs.notify_period;
    this.date_delayed_to = attrs.date_delayed_to;
    this.created_at = attrs.created_at;
    this.updated_at = attrs.updated_at;
    this.closed_at = attrs.closed_at;
    this.email = attrs.email;
    this.creator_formed = attrs.creator_formed;
    this.creator_user = attrs.creator_user;
    this.executor_user = attrs.executor_user;
    this.type = attrs.type;
    this.status = attrs.status;
    this.status_color = attrs.status_color;
    this.status_transitions = attrs.status_transitions;
    this.queue = attrs.queue;
    this.priority = attrs.priority;
    this.dynamic_fields = attrs.dynamic_fields;
    this.queue_topic = attrs.queue_topic;
    this.time_elapsed = attrs.time_elapsed;
    this.time_eta = attrs.time_eta;
    this.time_sla = attrs.time_sla;
    this.date_eta = attrs.date_eta;
    this.files = attrs.files
    this.linked_tickets = attrs.linked_tickets;
    this.children = attrs.children;
    this.link_comment = attrs.link_comment;
    this.history = attrs.history;
    this.current_user_permissions = attrs.current_user_permissions;
    this.ticket_otrs = attrs.ticket_otrs;
    this.ticket_invoices = attrs.ticket_invoices;
    this.user_favorite_id = attrs.user_favorite_id;
    this.user_review_id = attrs.user_review_id;
    this.criticality = attrs.criticality;
    this.theme = attrs.theme;
    this.interaction_type = attrs.interaction_type;
    this.is_private = attrs.is_private;
    this.reminders = attrs.reminders ?? []
    this.disallow_reopen = attrs.disallow_reopen;
    this.visible_to_client = attrs.visible_to_client;
  }

  async addComment(ticketComment: ITicketCommentUpdateForm): Promise<ITicketComment> {
    return await Ticket.ticketCommentStore.createRow(ticketComment);
  }

  static isTicketEditable(ticket: ITicket | ITicketForList | ITicketForMassActions): boolean {
    return [IStatus.New, IStatus.Processing, IStatus.Delayed].includes(ticket.status);
  }

  static isTicketClosed(ticket: ITicket | ITicketForList | ITicketForMassActions): boolean {
    return [IStatus.Closed, IStatus.Canceled].includes(ticket.status);
  }

  static isTicketCreator(ticket: ITicket, userId: number | string): boolean {
    return ticket.creator_user?.id === +userId
  }

  static isTicketExecutor(ticket: ITicket, userId: number | string): boolean {
    return ticket.executor_user?.id === +userId
  }

  static fromCustomerTypeToCommentableType(customerType: ITicket['customer_type']): RequestCommentableType | null {
    if (!customerType) {
      return null
    }

    switch (customerType) {
      case TicketCustomerTypes.CLIENT:
      case TicketCustomerTypes.PRODUCT_PAYPORT_MERCHANT:
      case TicketCustomerTypes.PRODUCT_PAYPORT_AGENT: return RequestCommentableType.CLIENT
      case TicketCustomerTypes.LEAD: return RequestCommentableType.LEAD
      case TicketCustomerTypes.TRADER: return RequestCommentableType.TRADER
      case TicketCustomerTypes.SERVICE_PROVIDER: return RequestCommentableType.SERVICE_PROVIDER

      default: return null;
    }
  }
}

export {Ticket};
