import {Ref, ref, toRefs, nextTick} from 'vue'
import {ITicketCommentUpdateForm} from "@/pages/ts/ticket_comments/interfaces";
import {DropzoneFile} from "dropzone";
import {maxLength, required, requiredIf} from "@/utils/i18n-validators";
import useVuelidate, {ServerErrors} from "@vuelidate/core";
import {AxiosResponse} from "axios";

export const useFormData = () => {
  const formDataComment: Ref<ITicketCommentUpdateForm> = ref({
    ticket_id: 0,
    text: '',
    status: 0,
    files: [] as { name: string, hash: string, file: DropzoneFile } [],
    isLoading: false,
    mail_account_id: '',
    recipient_email: [],
    send_email: false,
  } as ITicketCommentUpdateForm);
  const $externalResults: Ref<ServerErrors> = ref({} as ServerErrors)
  const checkedDuplicateReplyToEmail = ref(false)
  const listingIsOn: Ref<boolean> = ref(true)

  const reinitListing = async () => {
    listingIsOn.value = false;
    await nextTick();
    listingIsOn.value = true;
  };

  const backendErrorsComment: Ref<{
    [key: string]: {
      $error: boolean,
      $errors: Array<{
        $message: string,
      }>,
    }
  }> = ref({});

  const rulesComment = {
    ticket_id: {required},
    status: {},
    text: {
      maxLengthVerbose: maxLength(5000),
    },
    files: {},
    mail_account_id: {requiredIf: requiredIf(() => checkedDuplicateReplyToEmail.value)},
    recipient_email: {requiredAtLeastOneContact: requiredIf(() => checkedDuplicateReplyToEmail.value)},
    send_email: {requiredIf: requiredIf(() => checkedDuplicateReplyToEmail.value)},
  };

  const validateComment = useVuelidate<ITicketCommentUpdateForm>(
    rulesComment,
    toRefs(formDataComment.value),
    {
      $externalResults,
      $lazy: true,
      $scope: false,
    }
  );

  const commentStartFormDataLoading = () => {
    formDataComment.value.isLoading = true;
  }

  const commentStopFormDataLoading = () => {
    formDataComment.value.isLoading = false;
  }

  const updateCommentExternalResults = (errors: ServerErrors) => {
    $externalResults.value = errors;
  }

  const updateCommentBackendResults = (el: string, response: AxiosResponse) => {
    backendErrorsComment.value[el] = {
      $error: true,
      $errors: response.data.errors[el].map((el: string) => ({$message: el})),
    }
  }

  const clearCommentExternalResults = () => {
    validateComment.value.$clearExternalResults();
  }
  const clearCommentBackendResults = () => {
    backendErrorsComment.value = {};
  }

  const flushFormDataComment = () => {
    formDataComment.value.text = '';
    formDataComment.value.files = [];
  }

  const saveEditedAccount = (accountId: string) => {
    formDataComment.value.mail_account_id = accountId
  };

  const saveEditedEmail = (index: number, value: string | string[]) => {
    formDataComment.value.recipient_email[index] = value as string;
  };

  const removeRowEmail = async (index: number) => {
    formDataComment.value.recipient_email.splice(index, 1);
    await reinitListing();
  };

  const addRowEmail = () => {
    formDataComment.value.recipient_email.push('');
  };

  return {
    listingIsOn,
    formDataComment,
    validateComment,
    checkedDuplicateReplyToEmail,

    saveEditedAccount,

    clearCommentExternalResults,
    clearCommentBackendResults,

    updateCommentBackendResults,
    updateCommentExternalResults,

    flushFormDataComment,
    saveEditedEmail,
    removeRowEmail,
    addRowEmail,

    commentStartFormDataLoading,
    commentStopFormDataLoading,
  }
}