import {
  IImportTicketInvoiceFormData, IParticipant,
  ITicketInvoice,
  TicketInvoiceCompensationSideable,
  TicketInvoiceImportStatuses,
  TicketInvoiceParticipantable
} from "@/pages/ts/tickets/interfaces";
import {useRouter} from "vue-router";
import {useI18n} from "vue-i18n";
import {Icon} from "@/base-components/Lucide/Lucide.vue";
import {IFile} from "@/stores/file/interfaces";

export function ticketInvoiceParticipatableMapper(type?: TicketInvoiceParticipantable): string {
  if (!type) {
    return ""
  }

  switch (type) {
    case TicketInvoiceParticipantable.MERCHANT:
      return 'Merchant'
    case TicketInvoiceParticipantable.TRADER:
      return 'Trader'
    case TicketInvoiceParticipantable.SERVICE_PROVIDER:
      return 'Service provider'
    default:
      throw new Error('Invoice participant is not supported')
  }
}

export function getCompensationOptions(ticketInvoice: ITicketInvoice) {
  const {t} = useI18n();

  return [
    {
      text: ticketInvoice.claimant?.participantable?.name ?? '',
      value: getCompensationValue(ticketInvoice, TicketInvoiceCompensationSideable.CLAIMANT),
    },
    {
      text: ticketInvoice.defendant?.participantable?.name ?? '',
      value: getCompensationValue(ticketInvoice, TicketInvoiceCompensationSideable.DEFENDANT),
    },
    {
      text: 'Payport',
      value: getCompensationValue(ticketInvoice, TicketInvoiceCompensationSideable.PAYPORT),
    },
    {
      text: t('ts.tickets.Without compensation'),
      value: getCompensationValue(ticketInvoice, TicketInvoiceCompensationSideable.WITHOUT),
    },
  ]
}

export function getCompensationValue(ticketInvoice: ITicketInvoice, type: TicketInvoiceCompensationSideable) {
  const getId = () => {
    if (type === TicketInvoiceCompensationSideable.PAYPORT) {
      return null
    }

    return type === TicketInvoiceCompensationSideable.CLAIMANT
      ? ticketInvoice.claimant?.id
      : ticketInvoice.defendant?.id
  }

  return JSON.stringify({
    'sideable_id': getId(),
    'sideable_type': type,
    'ticket_invoice_id': ticketInvoice.id,
  })
}

export function participantLink(participant: IParticipant | undefined | null) {
  if (!participant) {
    return ''
  }

  try {
    return getLink(participant)
  }
  catch {
    return ''
  }
}

function getLink(participant: IParticipant) {
  const router = useRouter()

  switch (participant.participantable_type) {
    case TicketInvoiceParticipantable.MERCHANT:
      return router.resolve({
        name: 'clients.products.show', params: {
          clientId: participant.participantable?.client?.id,
          id: participant.participantable?.client?.pivot.product_id,
          section: 'main_info',
        }
      }).fullPath
    case TicketInvoiceParticipantable.SERVICE_PROVIDER:
      return router.resolve({
        name: 'service-providers.profile', params: {spId: participant.participantable.id}
      }).fullPath
    case TicketInvoiceParticipantable.TRADER:
      return router.resolve({
        name: 'traders.crmts', params: {traderId: participant.participantable.id}
      }).fullPath

    default:
      return '#'
  }
}

export function isLinkAvailable(participant: IParticipant | undefined | null) {
  if (!participant) {
    return false
  }

  try {
    getLink(participant)
  }
  catch {
    return false
  }

  if (!isClaimantOrDefendantCompensation(participant.sideable_type)) {
    return false
  }

  if (participant.participantable_type !== TicketInvoiceParticipantable.SERVICE_PROVIDER) {
    return true
  }

  return participant.participantable?.type === 'trader'
}

export function compensationName(ticketInvoice: ITicketInvoice): string {
  if (!ticketInvoice || !ticketInvoice.compensation) {
    return ''
  }

  const {t} = useI18n()

  if (ticketInvoice.compensation.sideable.sideable_type === TicketInvoiceCompensationSideable.PAYPORT) {
    return t('Payport')
  }

  if (ticketInvoice.compensation.sideable.sideable_type === TicketInvoiceCompensationSideable.WITHOUT) {
    return t('ts.tickets.Without compensation')
  }

  return ticketInvoice.compensation.sideable.participantable.name
}


export function compensationType(ticketInvoice: ITicketInvoice): string {
  if (!isClaimantOrDefendantCompensation(ticketInvoice?.compensation?.sideable.sideable_type)) {
    return ''
  }

  const {t} = useI18n()

  return t('ts.tickets.' + ticketInvoiceParticipatableMapper(ticketInvoice.compensation?.sideable.participantable_type))
}

export function isClaimantOrDefendantCompensation(sideable: TicketInvoiceCompensationSideable | undefined) {
  return sideable === TicketInvoiceCompensationSideable.CLAIMANT || sideable === TicketInvoiceCompensationSideable.DEFENDANT
}

export function getTransactionShowIdKeyByType(participantable: TicketInvoiceParticipantable): 'merchant_id' | 'service_provider_id' | 'trader_id' {
  switch (participantable) {
    case TicketInvoiceParticipantable.MERCHANT:
      return 'merchant_id'
    case TicketInvoiceParticipantable.SERVICE_PROVIDER:
      return 'service_provider_id'
    case TicketInvoiceParticipantable.TRADER:
      return 'trader_id'
  }
}


export function iconMapper(ticketInvoice: IImportTicketInvoiceFormData): Icon | undefined {
  switch (ticketInvoice.status) {
    case TicketInvoiceImportStatuses.PENDING:
      return 'Clock'
    case TicketInvoiceImportStatuses.READY:
      return 'CheckCircle'
    case TicketInvoiceImportStatuses.LOADED:
      return 'Save'
    case TicketInvoiceImportStatuses.ERROR:
      return 'AlertCircle'
  }
}

export function colorTextMapper(ticketInvoice: IImportTicketInvoiceFormData): string {
  switch (ticketInvoice.status) {
    case TicketInvoiceImportStatuses.PENDING:
      return '!text-pending'
    case TicketInvoiceImportStatuses.READY:
      return 'text-primary'
    case TicketInvoiceImportStatuses.PROCESS:
      return '!text-blue-500'
    case TicketInvoiceImportStatuses.LOADED:
      return '!text-success'
    case TicketInvoiceImportStatuses.ERROR:
      return '!text-danger'
  }
}

export function colorRowMapper(status: TicketInvoiceImportStatuses): string {
  switch (status) {
    case TicketInvoiceImportStatuses.READY:
      return 'bg-blue-100'
    case TicketInvoiceImportStatuses.PROCESS:
      return '!bg-yellow-100'
    case TicketInvoiceImportStatuses.LOADED:
      return '!bg-green-100'
    case TicketInvoiceImportStatuses.ERROR:
      return '!bg-red-100'
    default:
      return ''
  }
}

export function messageMapper(ticketInvoice: IImportTicketInvoiceFormData): string {
  const {t} = useI18n()

  switch (ticketInvoice.status) {
    case TicketInvoiceImportStatuses.PENDING:
      return t('ts.tickets.Pending for load')
    case TicketInvoiceImportStatuses.READY:
      return t('ts.tickets.Success to load')
    case TicketInvoiceImportStatuses.PROCESS:
      return t('ts.tickets.Load in process')
    case TicketInvoiceImportStatuses.LOADED:
      return t('ts.tickets.Loaded')
    case TicketInvoiceImportStatuses.ERROR:
      return ticketInvoice.error
  }
}

export function getFileUrl(file: IFile): string {
  return new URL(file.file_url as string).pathname
}

export type FlatMapItem = {id: number, children: FlatMapItem[]}
export const flatMap = (arr = [] as FlatMapItem[], supervisor?: number) =>
  arr.reduce((list, { children, ...e }) => {
    /// @ts-ignore
    list.push(supervisor ? { supervisor, ...e } : e);
    if (children) list.push(...flatMap(children, e.id));

    return list;
  }, []);

