import axios from "axios";
import Echo from "laravel-echo";
import Pusher, { Channel } from "pusher-js";
import { metaEnv } from "@/utils/helper";
import { i18n } from "@/utils/i18n";
import { useNotificationStore } from "@/stores/notification/notification";

import { INotificationWs, INotificationWithStyle } from "@/stores/notification/interfaces";

import { showNotification } from "@/mixins/ShowNotification.v2";
import { NotificationEvent, FrontNotificationType, SubscriberType } from "@/stores/notification/enums";
import { useUserStore } from "@/stores/user";
import {IDeferredExport} from "@/stores/deferred_export/interfaces";
import {useDeferredExport} from "@/stores/deferred_export/deferred_export";

interface CustomWindow extends Window {
  Pusher: typeof Pusher,
  Echo: Echo,
}

type SubscriptionMapping = {
  [ key: string ]: {
    subscriberTypes: Array<SubscriberType>,
    callback: (message: INotificationWs | INotificationWithStyle) => void,
  }
};

const { t } = i18n.global;

const subscriptionMapping: SubscriptionMapping = {
  [NotificationEvent.CommunicationReminderEvent]: {
    subscriberTypes: [ SubscriberType.USER ],
    callback: (message) => {
      message = message as INotificationWs;
      showNotification({
        type: FrontNotificationType.WARNING,
        title: message.title as string,
        content: `${message.datetime} ${message.content}`,
        persist: true,
        withSound: true,
      });
      useNotificationStore().hasNewNotifications = true;
    },
  },
  [NotificationEvent.WebGenericNotificationEvent]: {
    subscriberTypes: [ SubscriberType.USER, SubscriberType.ROLE, SubscriberType.DEPARTMENT ],
    callback: (message) => {
      message = message as INotificationWithStyle;
      if (useUserStore().auth.user.disabled_web_notifications?.includes(message.schedule_id)) return;

      showNotification({
        type: FrontNotificationType.CUSTOM,
        icon: message.icon,
        color: message.color,
        title: message.title as string,
        content: message.content,
        persist: true,
        withSound: true,
      });
      useNotificationStore().hasNewNotifications = true;
    }
  }
};

const localWindow = window as unknown as CustomWindow;
const appKeyAdded = !!import.meta.env.VITE_PUSHER_APP_KEY;

const subscribe = (params: {
  user_id: string | number,
  role_id?: Array<string | number>,
  department_id?: Array<string | number>,
}) => {
  if (!appKeyAdded) return;

  localWindow.Echo.leaveAllChannels();
  Object.keys(subscriptionMapping).forEach((eventName) => {
    const mappingEl = subscriptionMapping[eventName];

    if (mappingEl.subscriberTypes.includes(SubscriberType.USER)) {
      localWindow.Echo
        .private(`App.Models.User.${params.user_id}`)
        .listen(eventName, mappingEl.callback);

      localWindow.Echo
        .private('deferred.export.' + params.user_id)
        .listen('DeferredExportEvent', async function(data: {export: IDeferredExport, in_progress: boolean}) {
          useDeferredExport().hasDeferredExportInProgress = data.in_progress
        })
    }

    if (mappingEl.subscriberTypes.includes(SubscriberType.ROLE)) {
      params.role_id?.forEach((id) => {
        localWindow.Echo
          .private(`App.Models.Role.${id}`)
          .listen(eventName, mappingEl.callback);
      });
    }

    if (mappingEl.subscriberTypes.includes(SubscriberType.DEPARTMENT)) {
      params.department_id?.forEach((id) => {
        localWindow.Echo
          .private(`App.Models.Department.${id}`)
          .listen(eventName, mappingEl.callback);
      });
    }
  })

};

const unsubscribe = () => {
  if (!appKeyAdded) return;

  localWindow.Echo.leaveAllChannels();
};

export default () => {
  if (!appKeyAdded) return;

  localWindow.Pusher = Pusher;
  localWindow.Echo = new Echo({
    broadcaster: "pusher",
    key: import.meta.env.VITE_PUSHER_APP_KEY,
    cluster: import.meta.env.VITE_PUSHER_APP_CLUSTER,
    wsHost: metaEnv.VITE_PUSHER_WS_HOST,
    wsPort: import.meta.env.VITE_PUSHER_WS_PORT,
    authEndpoint: metaEnv.VITE_APP_API_URL + "/broadcasting/auth",
    authorizer: (channel: Channel) => {
      return {
        authorize: async (socketId: string, callback: (failed: boolean, data: object) => void) => {
          try {
            const res = await axios.post("/api/broadcasting/auth", {
              socket_id: socketId,
              channel_name: channel.name,
            });
            callback(false, res.data);
          } catch (err) {
            const error = err as object;
            callback(true, error);
          }
        }
      }
    },
    forceTLS: import.meta.env.VITE_PUSHER_FORCE_TLS ? !!parseInt(import.meta.env.VITE_PUSHER_FORCE_TLS) : false,
    disableStats: true,
    enabledTransports: ["ws", "wss"],
  });
};

export const usePusher = () => {
  return {
    subscribe,
    unsubscribe,
    Echo: localWindow.Echo,
  };
};
