import { defineStore } from "pinia";
import { RouteLocationNormalized } from "vue-router";
import { RouteLocationMatched } from "vue-router";
import axios, { AxiosError } from "axios";
import _ from "lodash";

export interface IBreadcrumb {
  title: string,
  url: string,
  dynamic?: boolean,
  idParam?: string,
  crumbApi?: string,
  shouldTrim?: boolean,
}

interface IPredefinedData {
  id: string,
  value: string,
}

interface IPredefined {
  [key: string]: Array<IPredefinedData>,
}

export const useBreadcrumbsStore = defineStore("breadcrumbs", {
  state: () => {
    return {
      crumbs: [] as Array<IBreadcrumb>,
      predefined: {} as IPredefined,
    };
  },
  actions: {
    setCrumbs(currentRoute: RouteLocationNormalized) {
      const router = this.$router;

      this.crumbs = [];

      const routeParams = currentRoute.params;
      currentRoute.matched
        .filter(function (el: RouteLocationMatched) { return !!el.meta && !!el.meta.breadcrumb })
        .map(async (el: RouteLocationMatched, index) => {
          const breadcrumb = el.meta.breadcrumb as IBreadcrumb;
          const newCrumb = {} as IBreadcrumb;

          let path = "";
          const resolved = router.resolve(el.path);
          if (Object.keys(resolved.params).length > 0) {
            resolved.params = {
              ...resolved.params,
              ...routeParams,
            };
            path = resolved.path;
            Object.keys(resolved.params).forEach(key => {
              path = path.replace(`:${key}`, resolved.params[key] as string);
            });
            newCrumb.url = encodeURI(path);
          } else {
            newCrumb.url = el.path;
          }

          if (breadcrumb.dynamic && breadcrumb.idParam && breadcrumb.crumbApi) {
            const id = routeParams[breadcrumb.idParam] as string;

            let filledFromPredefined = false;
            let predefinedByRoute = _.get(this.predefined, breadcrumb.crumbApi);

            if (predefinedByRoute) {
              const predefined = predefinedByRoute.filter(el => el.id == id);
              if (predefined.length > 0) {
                newCrumb.title = predefined[0].value;
                filledFromPredefined = true;
              }
            }

            if (!filledFromPredefined) {
              try {
                const res = await axios.get(breadcrumb.crumbApi, { params: { id } });
                newCrumb.title = res.data.value;

                if (!predefinedByRoute) {
                  this.predefined[breadcrumb.crumbApi] = [];
                  predefinedByRoute = this.predefined[breadcrumb.crumbApi];
                }
                predefinedByRoute.push({
                  id,
                  value: res.data.value,
                });
              } catch (err) {
                const error = err as AxiosError;
                console.error(error.response);
              }
            }
          } else {
            newCrumb.title = breadcrumb.title;
          }

          if (newCrumb.title !== this.crumbs[this.crumbs.length - 1]?.title) {
            this.crumbs.splice(index, 0, newCrumb);
          }
        });
    },
    clearCache(key: string) {
      this.predefined[key] = [];
    },
    updateCrumbs(crumbs: Array<IBreadcrumb>) {
      this.crumbs = crumbs;
    },
  },
});
