import { ref } from "vue";
import {IFilterData, IPage} from "@/utils/interfaces";
import { IClient, IClientForList, IClientUpdateForm } from "@/pages/clients/interfaces";
import {ICoefficient} from "@/pages/coefficients/interfaces";
import { IUserShort } from "@/pages/users/interfaces";
import { Client } from "@/pages/clients/classes";
import { defineStore } from "pinia";
import axios from "@/axios";
import { AxiosError } from "axios";

export const useClientStore = defineStore("clients", () => {
  const store = ref({
    responsibleUsersFilter: [] as Array<IUserShort>,
    coefficientsFilter: [] as Array<ICoefficient>,
  })

  async function getAllClients(params?: {
    ids?: Array<string>,
  }): Promise<Array<IClient>> {
    try {
      const res = await axios.get("/api/clients/all", { params });
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getClients(params: IFilterData = {}): Promise<IPage<IClientForList>> {
    try {
      const res = await axios.get("/api/clients", { params });
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getClient (id: number|string|null): Promise<Client> {
    try {
      const res = await axios.get("/api/clients/" + id);
      return new Client(res.data);
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateClient(id: number|string|null, data: IClientUpdateForm | { bind_contacts: Array<string> } | { unbind_contact: string }) {
    try {
      const res = await axios.put("/api/clients/" + id, data);
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getClientsPaginated(params: { page: number, perPage: number }): Promise<IPage<{ id: number, name: string, status_name: string }>>
  {
    try {
      const res = await axios.get("/api/clients/paginatedSelect", { params });
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getResponsibleUsersFilter(params?: {
    ids?: Array<string>,
  }) {
    try {
      const res = await axios.get("/api/clients/get-responsible-users-filter", { params });
      store.value.responsibleUsersFilter = res.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getCoefficientsFilter(params?: {
    ids?: Array<string>,
  }) {
    try {
      const res = await axios.get("/api/clients/get-coefficients-filter", { params });
      store.value.coefficientsFilter = res.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  return { store, getAllClients, getClients, getClient, updateClient, getClientsPaginated, getResponsibleUsersFilter, getCoefficientsFilter };
});
