import { defineStore } from "pinia";
import { computed, ref, Ref } from "vue";
import axios, { AxiosError } from "axios";
import dayjs from "dayjs";
import CustomParseFormat from "dayjs/plugin/customParseFormat";
dayjs.extend(CustomParseFormat);
import {
  ICommunicationParamsDate,
  ICommunicationData,
  ICommunicationType,
  ICommunicationForm,
  ICommunicationTaskPriority,
  ICommunication,
  ICommunicationCall,
  ICommunicationMeeting,
  ICommunicationOccupied,
} from "./interfaces";
import { useUserStore } from "../user";

interface ICommunicationRowStyle {
  [key: string]: {
    [key: string]: string,
  },
}

export const useCommunicationStore = defineStore("communications", () => {
  const user = useUserStore();
  const store = ref({
    taskPriority: {} as ICommunicationTaskPriority,
    bgColorMapping: {
      Call: {
        "2": "bg-gray-300 dark:bg-gray-700",
        "1": "bg-green-100 dark:bg-green-800",
        "0": "bg-red-100 dark:bg-red-900",
        "d": "bg-white dark:bg-darkmode-600", //default
      },
      Meeting: {
        "2": "bg-gray-300 dark:bg-gray-700",
        "1": "bg-green-100 dark:bg-green-800",
        "0": "bg-red-100 dark:bg-red-900",
        "d": "bg-white dark:bg-darkmode-600", // default
      },
      Task: {
        "4": "bg-gray-300 dark:bg-gray-700",
        "3": "bg-orange-100 dark:bg-orange-900",
        "2": "bg-green-100 dark:bg-green-800",
        "1": "bg-blue-200 dark:bg-blue-900",
        "0": "bg-white dark:bg-darkmode-600",
      }
    } as ICommunicationRowStyle,
    reminderVariants: [
      5, 10, 15, 20, 25, 30, 40, 50, 60
    ],
  });

  const getTaskPriority: Ref<ICommunicationTaskPriority> = computed(() => store.value.taskPriority);
  const getRowClasses = (comm: ICommunication): string => {
    const communication_type: string = comm.communication_type.name_original;
    let status: string = comm.communication_details.status;

    if (["Call", "Meeting"].indexOf(communication_type) != -1 
        && status == "0" 
        && dayjs().isBefore(dayjs((comm.communication_details as ICommunicationCall | ICommunicationMeeting).datetime, "DD.MM.YYYY HH:mm"))) {
      status = "d";
    }

    return store.value.bgColorMapping[communication_type][status];
  };
  const reminderVariants = computed(() => {
    return store.value.reminderVariants.map(el => el.toString());
  });
  
  async function getCommunications (params: ICommunicationParamsDate) : Promise<ICommunicationData> {
    try {
      const res = await axios.get("/api/communications", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCommunicationTypes () : Promise<Array<ICommunicationType>> {
    try {
      const res = await axios.get("/api/communication-types");
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getAllTaskPriority () : Promise<void> {
    try {
      const res = await axios.get("/api/priority", { params: { "type": "Task" } });
      store.value.taskPriority = res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function createCommunication (params: ICommunicationForm) {
    try {
      const res = await axios.post("/api/communications", params);
      await user.loadUserData();
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCommunication (id: string | number): Promise<ICommunication> {
    try {
      const res = await axios.get("/api/communications/" + id);
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function updateCommunication (id: string | number, data: ICommunicationForm): Promise<ICommunication> {
    try {
      const res = await axios.put("/api/communications/" + id, data);
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function deleteCommunication (id: string | number): Promise<void> {
    try {
      await axios.delete("/api/communications/" + id);
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function checkTimeOccupied (params: {
      datetime: string,
      duration?: string | number | null,
      id?: string | number | null,
  }): Promise<Array<ICommunicationOccupied>> {
    try {
      const res = await axios.get("/api/communications/check-time", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  return { 
    getAllTaskPriority,
    getTaskPriority,
    reminderVariants,
    getCommunications,
    getCommunicationTypes,
    createCommunication,
    getCommunication,
    updateCommunication,
    deleteCommunication,
    getRowClasses,
    checkTimeOccupied,
  };
});
