import {defineStore} from "pinia";
import {ref} from "vue";
import axios from "@/axios";
import {AxiosError} from "axios";
import {IDashboardCard, IWidgetCoordinate} from "@/pages/dashboard/interfaces";
import {isEmpty} from "lodash";

const defaultData = {
  currentUrl: '',
  dataUrl: '',
  name: '',
  data_source_type: 0
}



export const useWidgetStore = defineStore("widget", () => {
  const store = ref(defaultData);

  const dataSourceTypes = {
    TICKET: 0,
    LEADS: 1,
    CLIENTS: 2
  }

  function getData(){
    return store.value;
  }
  function setData(currentUrl: string, dataUrl: string, name: string, data_source_type: number){
    store.value.currentUrl = currentUrl;
    store.value.dataUrl = dataUrl;
    store.value.name = name;
    store.value.data_source_type = data_source_type
  }

  function clearData(){
    store.value = defaultData;
  }

  async function storeWidgetData(formData: IDashboardCard){
    try {
      const res = await axios.post('/api/widgets', formData);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  async function getAvailableWidgets(dashboard_id: string|null, additional_params={}){
    try {
      const res = await axios.get('/api/widgets', {params:{dashboard_id: dashboard_id, ...additional_params}});
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getAvailableDashboards(personal=false, routeParams= {}){
    try {
      if(!isEmpty(routeParams)) {
        const res = await axios.get('/api/dashboards?only_personal='+personal, {params: {...routeParams, paginate:1}});
        return res.data.data;
      } else {
        const res = await axios.get('/api/dashboards?only_personal='+personal);
        return res.data.data;
      }
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  async function getAllDashboards(){
    try {
      const res = await axios.get('/api/dashboards/all');
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  async function getRolesOrUserDashboards(){
    try {
      const res = await axios.get('/api/dashboards/user_or_roles');
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getRoleDashboards(routeParams= {}){
    try {
      if(routeParams) {
        const res = await axios.get('/api/dashboards/role', {params: routeParams});
        return res.data.data;
      } else {
        const res = await axios.get('/api/dashboards/role');
        return res.data.data;
      }
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function storeDashboardData(formData: {
      default_positioning: Array<IWidgetCoordinate>,
      id: string,
      name: string,
      roles: Array<string>
  }){
    try {
      const res = await axios.post('/api/dashboard/store_dashboard', formData);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  async function showDashboard(dashboard_id: string)
  {
    try {
      const res = await axios.get('/api/dashboard/show/'+dashboard_id);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  async function getDashboardData(dashboard_id: string)
  {
    try {
      const res = await axios.get('/api/dashboard/get_dashboard/'+dashboard_id);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  async function deleteDashboard(dashboard_id: string)
  {
    try {
      const res = await axios.delete('/api/dashboard/delete_dashboard/'+dashboard_id);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function loadWidgetData(widget_id: string)
  {
    try {
      const res = await axios.get('/api/widgets/'+widget_id);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function destroyWidget(widget_id: string)
  {
    try {
      const res = await axios.delete('/api/widgets/'+widget_id);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  async function updateWidgetData(widget_id: string, formData: IDashboardCard) {
    try {
      const res = await axios.put('/api/widgets/'+widget_id, formData);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateAttachRole(formData: {roles: Array<string>, dashboard_id: string}){
    try {
      const res = await axios.post('/api/dashboard/attach_role', formData);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  async function newDashboard(formData: {
      default_positioning: Array<IWidgetCoordinate>,
      name: string,
      type: string,
      roles: Array<string>
  }){
    try {
      const res = await axios.post('/api/dashboard/new', formData);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  async function updateDashboard(dashboard_id: string, formData: {name: string}){
    try {
      const res = await axios.post('/api/dashboard/update/'+dashboard_id, formData);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function setDefaultDashboard(dashboard_id: string){
    try {
      const res = await axios.post('/api/dashboard/set_default/'+dashboard_id);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  async function deleteDefaultDashboard(){
    try {
      const res = await axios.delete('/api/dashboard/delete_default/');
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  return {
    getData,
    setData,
    clearData,
    storeWidgetData,
    updateWidgetData,
    getAvailableWidgets,
    storeDashboardData,
    deleteDashboard,
    newDashboard,
    updateDashboard,
    getDashboardData,
    showDashboard,
    loadWidgetData,
    destroyWidget,
    getAvailableDashboards,
    getAllDashboards,
    getRolesOrUserDashboards,
    getRoleDashboards,
    updateAttachRole,
    setDefaultDashboard,
    deleteDefaultDashboard,
    store,
    dataSourceTypes
  }
});
