import { ref } from "vue";
import { defineStore } from "pinia";
import axios, { AxiosError } from "axios";
import { IPage } from "@/utils/interfaces";
import { IFile } from "@/stores/file/interfaces";

export const useFileStore = defineStore("files", () => {
  async function getFiles(params?: {
    fileable_type?: string,
    fileable_id?: number | string,
    perPage?: string | number | undefined,
    page?: string | number | undefined
  }): Promise<IPage<IFile>> {
    try {
      const res = await axios.get("/api/files", { params });
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateFile (id: string|number|null, data: { description: string }) {
    try {
      await axios.put("/api/files/" + id, data);
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function deleteFile (id: string|number|null) {
    try {
      await axios.delete("/api/files/" + id);
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function downloadFile (id: string|number|null, fileName: string) {
    try {
      const res = await axios({
        url: "/api/files/" + id,
        method: "GET",
        responseType: "arraybuffer"
      });
      const url = window.URL.createObjectURL(new Blob([res.data]));
      const link = document.createElement('a');
      link.href = url;
      link.setAttribute('download', fileName);
      document.body.appendChild(link);
      link.click();
      link.remove();
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function fetchBlob (id: string|number): Promise<ArrayBuffer> {
    try {
      const response = await axios({
        url: "/api/files/" + id,
        method: "GET",
        responseType: "arraybuffer"
      });

      return response.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function storeFile(params: {
    file: File,
    fileable_type: string,
    fileable_id: string | number,
    name: string,
    description?: string,
  }) {
    const formData = new FormData();
    Object.keys(params).forEach(key => {
      formData.append(key, params[key as keyof typeof params] as string | Blob);
    });
    try {
      const res = await axios.post(
        "/api/files",
        formData,
        {
          headers: {
            "Content-Type": "multipart/form-data",
          },
        },
      );
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  return { getFiles, updateFile, deleteFile, downloadFile, storeFile, fetchBlob };
});
