import { ref } from "vue";
import { defineStore } from "pinia";
import { IUserShort } from "@/pages/users/interfaces";
import { IAssociationType } from "@/stores/association/interfaces";
import { IPage } from "@/utils/interfaces";
import { ILead, ILeadBase, ILeadForm, ILeadShort } from "@/pages/leads/interfaces";
import { IFilterData } from "@/utils/interfaces";
import axios, { AxiosError } from "axios";
import { Lead } from "@/pages/leads/classes";
import { IClient } from "@/pages/clients/interfaces";
import { useUserStore } from "@/stores/user";

export const useLeadStore = defineStore('leads', () => {
  const user = useUserStore();
  const store = ref({
    responsibleUsersFilter: [] as Array<IUserShort>,
    associationTypesFilter: [] as Array<IAssociationType>,
    clientProductOrganisationTypesFilter: [] as Array<{ id: number, name: string }>
  })

  async function getLeads(params: IFilterData): Promise<IPage<ILead>> {
    try {
      const res = await axios.get("/api/leads", { params });
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getLead (id: number|string|null): Promise<Lead> {
    try {
      const res = await axios.get("/api/leads/" + id);
      return new Lead(res.data);
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function createLead(params: ILeadForm): Promise<ILead> {
    try {
      const res = await axios.post("/api/leads", params);
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateLead(id: number|string|null, data: ILeadBase | { bind_contacts: Array<string> } | { unbind_contact: string }) {
    try {
      await axios.put("/api/leads/" + id, data);
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function convertLead(id: number|string|null): Promise<IClient> {
    try {
      const res = await axios.post("/api/leads/" + id + "/convert");
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getAllLeads(): Promise<Array<IClient>> {
    try {
      const res = await axios.get("/api/leads/all");
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getLeadsByIds (params?: {
    ids?: Array<string>,
  }): Promise<Array<ILeadShort>> {
    try {
      const res = await axios.get("/api/leads/by-ids", { params });
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getResponsibleUsersFilter(params?: {
    ids?: Array<string>,
  }) {
    try {
      const res = await axios.get("/api/leads/get-responsible-users-filter", { params });
      store.value.responsibleUsersFilter = res.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getAssociationTypesFilter(params?: {
    ids?: Array<string>,
  }) {
    try {
      const res = await axios.get("/api/leads/get-association-types-filter", { params });
      store.value.associationTypesFilter = res.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  async function getClientProductOrganizationTypesFilter(params?: {
        ids?: Array<string>,
    }) {
    try {
      const res = await axios.get("/api/leads/get-client-product-organisation-types-filter", { params });
      store.value.clientProductOrganisationTypesFilter = res.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }
  return {
    store,
    getResponsibleUsersFilter,
    getAssociationTypesFilter,
    getClientProductOrganizationTypesFilter,
    getLeads,
    getLead,
    createLead,
    updateLead,
    convertLead,
    getAllLeads,
    getLeadsByIds,
  };
})
