import {IFilterData, IPage, ISelect} from "@/utils/interfaces";
import {
  IProductType,
  IProduct,
  ProductFormData,
  IPayportMerchant,
  IPayportAgent,
  IProductsImport,
  IGnuVpnClient,
} from "@/pages/products/interfaces";
import {defineStore} from "pinia";
import axios from "@/axios";
import downloadFile from "@/mixins/DownloadFile";
import {AxiosError} from "axios";
import {computed, ref} from "vue";

export const useProductStore = defineStore("product", () => {
  const store = ref({
    error: {} as AxiosError,
    selectMerchantsList: [] as (ISelect & {id: number})[],
  });

  const getError = computed(() => store.value.error);
  const getSelectMerchantsList = computed(() => store.value.selectMerchantsList);

  async function getAllProducts(): Promise<Array<IProduct<IPayportMerchant | IPayportAgent | IGnuVpnClient>>> {
    try {
      const res = await axios.get("/api/products");
      return res.data;
    } catch (error: any) {
      store.value.error = error;
      throw error.response;
    }
  }

  async function getProducts(params: IFilterData = {}): Promise<IPage<IProduct<IPayportMerchant | IPayportAgent | IGnuVpnClient>>> {
    try {
      const res = await axios.get("/api/products", {params});
      return res.data;
    } catch (error: any) {
      store.value.error = error;
      throw error.response;
    }
  }

  async function getProductsToImport(params: IFilterData = {}): Promise<IProductsImport> {
    try {
      const res = await axios.get("/api/products/import", {params});
      return res.data;
    } catch (error: any) {
      store.value.error = error;
      throw error.response;
    }
  }

  async function Import(data: object) {
    try {
      const res = await axios.post("/api/products/import", data);
      return res.data
    } catch (error: any) {
      store.value.error = error;
      throw error.response;
    }
  }

  async function updateProduct(id: number | string | null, data: object) {
    try {
      const res = await axios.put("/api/products/" + id, data);
      return res.data
    } catch (error: any) {
      store.value.error = error;
      throw error.response;
    }
  }

  async function getProductTypes(onlyPayport = false, client_id = ""): Promise<IPage<IProductType>> {
    try {
      const res = await axios.get("/api/product_types", {
        params: {
          onlyPayport: onlyPayport ? 1 : 0,
          client_id: client_id
        }
      });
      return res.data;
    } catch (error: any) {
      store.value.error = error;
      throw error.response;
    }
  }

  async function createProduct(data: ProductFormData) {
    try {
      const res = await axios.post("/api/products/create", data);
      return res.data;
    } catch (error: any) {
      store.value.error = error;
      throw error.response;
    }
  }

  async function getProductAdditionalData(params: IFilterData = {}) {
    try {
      if (!params.btn_export) {
        const res = await axios.get("/api/products/get_additional_info", {params});
        return res.data;
      } else {
        await downloadFile("/api/products/get_additional_info", params);
      }
    } catch (error: any) {
      store.value.error = error;
      throw error.response;
    }
  }

  async function getTransactionFilter(product_id: number | string) {
    try {
      const res = await axios.get("/api/products/get_additional_info?id=" + product_id + "&section=transactions_filter");
      return res.data;
    } catch (error: any) {
      store.value.error = error;
      throw error.response;
    }
  }

  async function getMerchantBalanceFilter(product_id: number | string) {
    try {
      const res = await axios.get("/api/products/get_additional_info?id=" + product_id + "&section=merchant_balance_filter");
      return res.data;
    } catch (error: any) {
      store.value.error = error;
      throw error.response;
    }
  }

  async function getMerchants(): Promise<Array<{ id: number, name: string }>> {
    try {
      const res = await axios.get('/api/products/merchants/all');

      store.value.selectMerchantsList = res.data.map((item: IPayportMerchant) => ({
        text: item.name,
        value: item.id,
        email: item.email,
      }))

      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getSandboxId(product_id: number|string): Promise<{ sandbox_id: Array<number> }> {
    try {
      const res = await axios.get('/api/products/get_merchant_sandbox_id/' + product_id);
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getAllProductsShort(params?: {
    ids?: Array<string>,
  }): Promise<Array<{ id: number, name: string }>> {
    try {
      const res = await axios.get("/api/products/short", { params });
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }


  return {
    getAllProducts,
    getProducts,
    getProductTypes,
    createProduct,
    getProductAdditionalData,
    getTransactionFilter,
    getMerchantBalanceFilter,
    getMerchants,
    getProductsToImport,
    Import,
    updateProduct,
    getSandboxId,
    getAllProductsShort,
    getError,
    getSelectMerchantsList,
  };
});
