import { defineStore } from "pinia";
import axios, { AxiosError } from "axios";
import { i18n } from "@/utils/i18n";

import {
  IReportPayportAuthLog,
  IReportPayportManagerActivity,
  IReportPayportWithNoActivity,
  IReportPayportAddedToBlacklist,
  IReportPayportMerchantsEnteringProd,
  IReportPayportMerchantsWithoutTransactions,
  IReportPayportAverageTimeToBargain,
  IReportPayportTopServiceProviders,
  IReportPayportGetServiceProvidersFilter,
  IReportPayportGetMerchantsFilter,
  IReportPayportGetFiatsFilter,
  IPageReportPayportTopMerchantsWithProfitability,
  IPageReportPayportTopMerchantsByIntervals,
  IReportPayportGetSalesManagersFilter,
  IReportPayportMerchantCurrency,
  IReportPayportGetTradersFilter,
  IReportPayportGetPaymentSystemsFilter,
  IReportPayportTraderConversions,
  IReportPayportCurrencyMerchant,
  IReportPayportMerchantRecalculations,
  IReportPayportTradersSpeed,
  IReportPayportMerchantsEnteringSand,
  IReportPayportAverageTimeToProdAfterSand,
  IReportPayportTraderCancellations,
  IReportPayportInvoicesDistributionByTime,
  IReportCRMLeadsByRespAndProducts,
  IReportCRMLeadConverting,
  IReportCRMHoldingParticipantLeads,
  IReportCRMAggregatorLeads,
  IReportCRMLeadsBySalesChannels,
  IReportCRMHoldingParticipantClients,
  IReportCRMAggregatorClients,
  IReportCRMDeactivatedClients,
  IReportCRMCustomerSupportActivity,
  IReportCrmEmployeeGrade,
  IReportCRMMerchantsStatusNew,
  IReportCRMCalendarTasks,
  IReportTSUserTicketsStatus,
  IReportPayportMerchantConversions,
  IReportTSUserTicketsByQueueStatus,
  IReportPayportMerchantCirculationAmountTimeline,
  IReportPayportFiatCirculationAmountTimeline,
} from "./interfaces";

import { RawMenuList } from "@/components/ExpandableMenu/utils";
import { IPage } from "@/utils/interfaces";
import {ITicketInvoice} from "@/pages/ts/tickets/interfaces";
import {capitalize} from "lodash";

const { t } = i18n.global;

export const useReportStore = defineStore("reports", () => {
  /* should contain relations between groups, report bindings go below */
  const reportGroups: RawMenuList = [
    {
      pageName: "reports.crm", title: t("Reports CRM"), subMenu: [
        { pageName: "reports.crm.personnel", title: t("Reports on personnel"),
          table: "reports_crm_personnel", action: "index", icon: "Library" },
        { pageName: "reports.crm.activities", title: t("Reports on activities"),
          table: "reports_crm_activities", action: "index", icon: "Library" },
        { pageName: "reports.crm.customer-support", title: t("Customer support department"),
          table: "reports_crm_customer_support", action: "index", icon: "Library" },
        { pageName: "reports.crm.employee-grade", title: t("Employee grade"),
          table: "reports_crm_employee_grade", action: "index", icon: "Library" },
      ], table: "reports_crm", action: "index", icon: "Boxes"
    },
    {
      pageName: "reports.payport", title: t("Reports Payport"), subMenu: [
        { pageName: "reports.payport.sales", title: t("Sales reports"),
          table: "reports_payport_sales", action: "index", icon: "Library" },
        { pageName: "reports.payport.maintenance", title: t("Maintenance reports"),
          table: "reports_payport_maintenance", action: "index", icon: "Library" },
        { pageName: "reports.payport.profitability", title: t("Profitability reports"),
          table: "reports_payport_profitability", action: "index", icon: "Library" },
        { pageName: "reports.payport.1-line", title: t("Reports - 1 line"),
          table: "reports_payport_1_line", action: "index", icon: "Library" },
      ], table: "reports_payport", action: "index", icon: "CreditCard"
    },
    {
      pageName: "reports.gnu-vpn", title: t("Reports GNU VPN"), subMenu: [],
      table: "reports_gnu_vpn", action: "index", icon: "ShieldCheck"
    },
    {
      pageName: "reports.crm_ts",
      title: t("Reports CRM_TS"),
      subMenu: [
        {
          pageName: "reports.crm_ts.ticket_invoices",
          title: t("Reports on ticket invoices"),
          table: "reports_crm_ts_ticket_invoices_general",
          action: "index",
          icon: "Library"
        },
        {
          pageName: "reports.crm_ts.user_tickets",
          title: t("Reports on ticket personnel"),
          table: "reports_crm_ts_user_tickets",
          action: "index",
          icon: "Library"
        },
      ],
      table: "reports_crm_ts",
      action: "index",
      icon: "ShieldCheck"
    },
  ];

  type ReportEntry = {
    routeName: string,
    name: string,
    isPersonalized: boolean,
    table?: string,
    action?: string,
  };
  type ReportGroupEntries = { [key: string]: Array<ReportEntry> };

  /* should contain only subgroup names as keys */
  const reportsBySubgroups: ReportGroupEntries = {
    "reports.crm.personnel": [
      { routeName: "reports.crm.personnel.auth-log", name: t("Auth log"), isPersonalized: false },
      { routeName: "reports.crm.personnel.manager-activity", name: t("Manager activity"), isPersonalized: false },
      { routeName: "reports.crm.personnel.limited-search", name: t("limited_search"), isPersonalized: false },
      { routeName: "reports.crm.personnel.calendar-tasks", name: t("Calendar tasks by managers"), isPersonalized: false},
    ],
    "reports.crm.activities": [
      { routeName: "reports.crm.activities.with-no-activity", name: t("Leads' and clients' activity"), isPersonalized: false },
      { routeName: "reports.crm.activities.added-to-blacklist", name: t("Added to blacklist"), isPersonalized: false },
      { routeName: "reports.crm.activities.merchants-entering-sand", name: t("Merchants entering sand"), isPersonalized: true },
      { routeName: "reports.crm.activities.payport-average-time-to-prod-after-sand", name: t("Average time to prod after sand"), isPersonalized: false },
      { routeName: "reports.crm.activities.merchants-entering-prod", name: t("Merchants entering prod"), isPersonalized: true },
      { routeName: "reports.crm.activities.merchants-without-transactions", name: t("Merchants without transactions"), isPersonalized: false },
      { routeName: "reports.crm.activities.average-time-to-bargain", name: t("Average time to bargain"), isPersonalized: false },
      { routeName: "reports.crm.activities.leads-by-resp-and-products", name: t("Leads by responsibles and products"), isPersonalized: false },
      { routeName: "reports.crm.activities.converted-cancelled-leads", name: t("Converted & cancelled leads"), isPersonalized: false },
      { routeName: "reports.crm.activities.holding-participant-leads", name: t("Holding leads by participants"), isPersonalized: false },
      { routeName: "reports.crm.activities.aggregator-leads", name: t("Aggregator leads"), isPersonalized: false },
      { routeName: "reports.crm.activities.leads-by-sales-channels", name: t("Leads by sales channels"), isPersonalized: false },
      { routeName: "reports.crm.activities.holding-participant-clients", name: t("Holding clients by participants"), isPersonalized: false },
      { routeName: "reports.crm.activities.aggregator-clients", name: t("Aggregator clients"), isPersonalized: false },
      { routeName: "reports.crm.activities.clients-by-sales-channels", name: t("Clients by sales channels"), isPersonalized: false },
      { routeName: "reports.crm.activities.deactivated-clients", name: t("Deactivated clients"), isPersonalized: false },
      { routeName: "reports.crm.activities.merchants-status-new", name: t("Merchants in status \"New\""), isPersonalized: false },
    ],
    "reports.crm.customer-support": [
      { routeName: "reports.crm.customer-support.customer-support-activity", name: t("Customer support department activity"), isPersonalized: true },
    ],
    "reports.crm.employee-grade": [
      { routeName: "reports.crm.employee-grade.employee-grade", name: t("Employee grade"), isPersonalized: false },
    ],
    "reports.payport.sales": [
      { routeName: "reports.payport.sales.top-currencies-without-profitability", name: t("Top currencies"), isPersonalized: false },
      { routeName: "reports.payport.sales.top-service-providers", name: t("Top service providers"), isPersonalized: false },
      { routeName: "reports.payport.sales.currency-merchant", name: t("Currency merchant"), isPersonalized: false },
      { routeName: "reports.payport.sales.merchant-currency", name: t("Merchant currency"), isPersonalized: false },
      { routeName: "reports.payport.sales.merchant-recalculations", name: t("Merchant recalculations"), isPersonalized: false },
      { routeName: "reports.payport.sales.trader-conversions", name: t("Trader conversions"), isPersonalized: false },
      { routeName: "reports.payport.sales.merchant-cancellations", name: t("Trader cancellations"), isPersonalized: false },
      { routeName: "reports.payport.sales.traders-speed", name: t("Traders speed"), isPersonalized: false },
      { routeName: "reports.payport.sales.invoices-distribution-by-time", name: t("Invoices distribution by time"), isPersonalized: false},
    ],
    "reports.payport.maintenance": [
      { routeName: "reports.payport.maintenance.top-merchants-without-profitability", name: t("Top merchants"), isPersonalized: true },
      { routeName: "reports.payport.maintenance.merchant-conversions", name: t("Merchant conversions"), isPersonalized: true },
      { routeName: "reports.payport.maintenance.merchant-circulation-amount-timeline", name: t("Merchant circulation amount timeline"), isPersonalized: false},
      { routeName: "reports.payport.maintenance.fiat-circulation-amount-timeline", name: t("Fiat circulation amount timeline"), isPersonalized: false},
    ],
    "reports.payport.profitability": [
      { routeName: "reports.payport.profitability.top-service-providers-with-profitability", name: t("Top service providers (profitability)"), isPersonalized: true },
      { routeName: "reports.payport.profitability.top-currencies-with-profitability", name: t("Top currencies (profitability)"), isPersonalized: true },
      { routeName: "reports.payport.profitability.top-merchants-with-profitability", name: t("Top merchants (profitability)"), isPersonalized: true },
      { routeName: "reports.payport.profitability.top-merchants-by-intervals", name: t("Top merchants by intervals (profitability)"), isPersonalized: true },
    ],
    "reports.payport.1-line": [
      { routeName: "reports.payport.1-line.trader-conversions", name: t("Trader conversions"), isPersonalized: false },
      { routeName: "reports.payport.1-line.merchant-conversions", name: t("Merchant conversions"), isPersonalized: false },
    ],
    "reports.crm_ts.ticket_invoices": [
      { routeName: "reports.crm_ts.ticket_invoices.general", name: capitalize(t("reports_crm_ts_ticket_invoices_general")), isPersonalized: false },
    ],
    "reports.crm_ts.user_tickets": [
      { routeName: "reports.crm_ts.user_tickets.status", name: capitalize(t("reports_crm_ts_user_tickets_status")), isPersonalized: false },
      {
        routeName: "reports.crm_ts.user_tickets_by_queue.status",
        name: capitalize(t("reports_crm_ts_user_tickets_by_queue_status")),
        table:  "reports_crm_ts_user_tickets_by_queue_status",
        action: "index",
        isPersonalized: false,
      },
    ],
  };

  const statusesList = [
    { text: t("Paid"), value: "1" },
    { text: t("Created"), value: "0" },
    { text: t("Confirmed"), value: "2" },
    { text: t("In process"), value: "4" },
    { text: t("Trader accepted"), value: "3" },
    { text: t("User selected"), value: "5" },
    { text: t("Hold"), value: "96" },
    { text: t("Canceled by revert"), value: "97" },
    { text: t("Canceled by timeout"), value: "98" },
    { text: t("Canceled"), value: "99" },
  ];

  const typesList = [
    { text: t("Sell"), value: "0" },
    { text: t("Buy"), value: "1" },
  ];

  const arbitrageList = [
    { text: t("In arbitrage"), value: "1" },
    { text: t("Not in arbitrage"), value: "2" },
  ];

  const stepList = [
    { text: t("Day"), value: "day" },
    { text: t("Week"), value: "week" },
    { text: t("Month"), value: "month" },
    { text: t("Year"), value: "year" },
  ];

  const amountTypesList = [
    { text: t("Not fiat"), value: "0" },
    { text: t("Fiat"), value: "1" },
    { text: t("Local methods"), value: "2" },
  ];

  const conversionRangesList = [
    { text: t("0-10%"), value: "0" },
    { text: t("10-20%"), value: "1" },
    { text: t("20-30%"), value: "2" },
    { text: t("30-40%"), value: "3" },
    { text: t("40-50%"), value: "4" },
    { text: t("50-60%"), value: "5" },
    { text: t("60-70%"), value: "6" },
    { text: t("70-80%"), value: "7" },
    { text: t("80-90%"), value: "8" },
    { text: t("90-100%"), value: "9" },
  ];

  const dayOfWeekList = [
    { text: t("Monday"), value: "1" },
    { text: t("Tuesday"), value: "2" },
    { text: t("Wednesday"), value: "3" },
    { text: t("Thursday"), value: "4" },
    { text: t("Friday"), value: "5" },
    { text: t("Saturday"), value: "6" },
    { text: t("Sunday"), value: "0" },
  ];

  const monthsList = [
    { text: t("January"), value: "1" },
    { text: t("February"), value: "2" },
    { text: t("March"), value: "3" },
    { text: t("April"), value: "4" },
    { text: t("May"), value: "5" },
    { text: t("June"), value: "6" },
    { text: t("July"), value: "7" },
    { text: t("August"), value: "8" },
    { text: t("September"), value: "9" },
    { text: t("October"), value: "10" },
    { text: t("November"), value: "11" },
    { text: t("December"), value: "12" },
  ];

  async function getPayportAuthLog (params: {
    date_begin?: string,
    date_end?: string,
    "responsible_user[]"?: Array<string>,
  }): Promise<IPage<IReportPayportAuthLog>>
  {
    try {
      const res = await axios.get("/api/reports/payport-auth-log", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportManagerActivity (params: {
    field_name?: string,
    field_value?: string,
    "responsible_user[]"?: Array<string>,
    date_begin?: string,
    date_end?: string,
    email?: string,
    type?: string,
    entity?: string,
  }): Promise<IPage<IReportPayportManagerActivity>>
  {
    try {
      const res = await axios.get("/api/reports/payport-manager-activity", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportWithNoActivity (params: {
    "responsible_user[]"?: Array<string>;
    "client[]"?: Array<string>,
    "lead[]"?: Array<string>,
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportWithNoActivity>>
  {
    try {
      const res = await axios.get("/api/reports/payport-with-no-activity", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportAddedToBlacklist (params: {
    "responsible_user[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportAddedToBlacklist>>
  {
    try {
      const res = await axios.get("/api/reports/payport-added-to-blacklist", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportMerchantsEnteringProd (params: {
    "responsible_user[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportMerchantsEnteringProd>>
  {
    try {
      const res = await axios.get("/api/reports/payport-merchants-entering-prod", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportAverageTimeToBargain (params: {
    "responsible_user[]"?: Array<string>;
    "merchant[]"?: Array<string>,
    bargain_qnt?: string,
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportAverageTimeToBargain>>
  {
    try {
      const res = await axios.get("/api/reports/payport-average-time-to-bargain", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportMerchantsWithoutTransactions (params: {
    "responsible_user[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportMerchantsWithoutTransactions>>
  {
    try {
      const res = await axios.get("/api/reports/payport-merchants-without-transactions", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportTopServiceProviders (params: {
    "status[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    "service_providers[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportTopServiceProviders>>
  {
    try {
      const res = await axios.get("/api/reports/payport-top-service-providers", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportTopServiceProvidersWithProfitability (params: {
    "status[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    "service_providers[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportTopServiceProviders>>
  {
    try {
      const res = await axios.get("/api/reports/payport-top-service-providers-with-profitability", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportGetServiceProvidersFilter (params?: {
    mode?: string;
    by_permission?: string
  }): Promise<Array<IReportPayportGetServiceProvidersFilter>>
  {
    try {
      const res = await axios.get("/api/reports/payport-get-service-providers-filter", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportTopCurrenciesWithProfitability (params: {
    "status[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    "sales_managers[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPageReportPayportTopMerchantsWithProfitability>
  {
    try {
      const res = await axios.get("/api/reports/payport-top-currencies-with-profitability", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportTopCurrenciesWithoutProfitability (params: {
    "status[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    "sales_managers[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPageReportPayportTopMerchantsWithProfitability>
  {
    try {
      const res = await axios.get("/api/reports/payport-top-currencies-without-profitability", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportGetMerchantsFilter (params?: {
    by_permission?: string
  }): Promise<Array<IReportPayportGetMerchantsFilter>>
  {
    try {
      const res = await axios.get("/api/reports/payport-get-merchants-filter", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportGetFiatsFilter (): Promise<Array<IReportPayportGetFiatsFilter>>
  {
    try {
      const res = await axios.get("/api/reports/payport-get-fiats-filter");
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportTopMerchantsWithProfitability (params: {
    "status[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    "sales_managers[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPageReportPayportTopMerchantsWithProfitability>
  {
    try {
      const res = await axios.get("/api/reports/payport-top-merchants-with-profitability", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportMerchantCirculationAmountTimeline (params: {
    "status[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    "sales_managers[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IReportPayportMerchantCirculationAmountTimeline>
  {
    try {
      const res = await axios.get("/api/reports/payport-merchant-circulation-timeline", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportFiatCirculationAmountTimeline (params: {
        "status[]"?: Array<string>;
        "merchants[]"?: Array<string>;
        "sales_managers[]"?: Array<string>;
        "fiats[]"?: Array<string>;
        date_begin?: string;
        date_end?: string;
    }): Promise<IReportPayportFiatCirculationAmountTimeline>
  {
    try {
      const res = await axios.get("/api/reports/payport-fiat-circulation-timeline", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportGetSalesManagersFilter (): Promise<Array<IReportPayportGetSalesManagersFilter>>
  {
    try {
      const res = await axios.get("/api/reports/payport-get-sales-managers-filter");
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportMerchantCurrency (params: {
    "status[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportMerchantCurrency>>
  {
    try {
      const res = await axios.get("/api/reports/payport-merchant-currency", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportCurrencyMerchant (params: {
    "status[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportCurrencyMerchant>>
  {
    try {
      const res = await axios.get("/api/reports/payport-currency-merchant", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportMerchantRecalculations (params: {
    "types[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportMerchantRecalculations>>
  {
    try {
      const res = await axios.get("/api/reports/payport-merchant-recalculations", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportMerchantsEnteringSand (params: {
    "responsible_user[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportMerchantsEnteringSand>>
  {
    try {
      const res = await axios.get("/api/reports/payport-merchants-entering-sand", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportGetTradersFilter (): Promise<Array<IReportPayportGetTradersFilter>>
  {
    try {
      const res = await axios.get("/api/reports/payport-get-traders-filter");
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportAverageTimeToProdAfterSand (params: {
    "responsible_user[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportAverageTimeToProdAfterSand>>
  {
    try {
      const res = await axios.get("/api/reports/payport-average-time-to-prod-after-sand", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportGetPaymentSystemsFilter (): Promise<Array<IReportPayportGetPaymentSystemsFilter>>
  {
    try {
      const res = await axios.get("/api/reports/payport-get-payment-systems-filter");
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportTraderConversions (params: {
    "service_providers[]"?: Array<string>;
    "traders[]"?: Array<string>;
    "types[]"?: Array<string>;
    "payment_systems[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportTraderConversions>>
  {
    try {
      const res = await axios.get("/api/reports/payport-trader-conversions", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportTraderConversionsSubquery (params: {
        "service_providers[]"?: Array<string>;
        "traders[]"?: Array<string>;
        "types[]"?: Array<string>;
        "payment_systems[]"?: Array<string>;
        date_begin?: string;
        date_end?: string;
        trader_id?: string;
        fiat_name?: string;
        payment_system_id?:string;
        payment_system_type?: string;
        invoice_type_subquery?: string
        service_provider_id?: string
    }): Promise<Array<IReportPayportTraderConversions>>
  {
    try {
      const res = await axios.get("/api/reports/payport-trader-conversions/subquery", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportMerchantConversions (params: {
    "service_providers[]"?: Array<string>;
    "traders[]"?: Array<string>;
    "types[]"?: Array<string>;
    "payment_systems[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportMerchantConversions>>
  {
    try {
      const res = await axios.get("/api/reports/payport-merchant-conversions", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportMerchantConversionsSubquery (params: {
        "service_providers[]"?: Array<string>;
        "traders[]"?: Array<string>;
        "types[]"?: Array<string>;
        "payment_systems[]"?: Array<string>;
        date_begin?: string;
        date_end?: string;
        merchant_id?: string;
        payment_system_id?:string;
        payment_system_type?: string;
        invoice_type_subquery?: string
  }): Promise<Array<IReportPayportMerchantConversions>>
  {
    try {
      const res = await axios.get("/api/reports/payport-merchant-conversions/subquery", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportTraderCancellations (params: {
    "service_providers[]"?: Array<string>;
    "traders[]"?: Array<string>;
    "types[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportTraderCancellations>>
  {
    try {
      const res = await axios.get("/api/reports/payport-trader-cancellations", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }
  async function getPayportTraderCancellationsSubquery (params: {
        "service_providers[]"?: Array<string>;
        "traders[]"?: Array<string>;
        "types[]"?: Array<string>;
        "payment_systems[]"?: Array<string>;
        date_begin?: string;
        date_end?: string;
        trader_id?: string;
        payment_system_id?:string;
        payment_system_type_id?: string;
        invoice_type_subquery?: string;
        fiat_name?: string;
    }): Promise<Array<IReportPayportTraderCancellations>>
  {
    try {
      const res = await axios.get("/api/reports/payport-trader-cancellations/subquery", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }
  async function getPayportTradersSpeed (params: {
    "service_providers[]"?: Array<string>;
    "traders[]"?: Array<string>;
    "types[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportTradersSpeed>>
  {
    try {
      const res = await axios.get("/api/reports/payport-traders-speed", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportInvoicesDistributionByTime (params: {
    "fiats[]"?: Array<string>;
    "types[]"?: Array<string>;
    "status[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    "service_providers[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPage<IReportPayportInvoicesDistributionByTime>>
  {
    try {
      const res = await axios.get("/api/reports/payport-invoices-distribution-by-time", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMLeadsByRespAndProducts (params: {
    "responsible_user[]"?: Array<string>,
    "product_type[]"?: Array<string>,
    "status[]"?: Array<string>,
    email?: string,
    date_begin?: string,
    date_end?: string,
  }): Promise<IPage<IReportCRMLeadsByRespAndProducts>>
  {
    try {
      const res = await axios.get("/api/reports/crm-leads-by-resp-and-products", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMConvertedCancelledLeads (params: {
    "responsible_user[]"?: Array<string>,
    "product_type[]"?: Array<string>
    filter_field?: string,
    filter_value?: string,
    date_begin?: string,
    date_end?: string,
  }): Promise<IPage<IReportCRMLeadConverting>>
  {
    try {
      const res = await axios.get("/api/reports/crm-converted-cancelled-leads", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMHoldingParticipantLeads (params: {
    "responsible_user[]"?: Array<string>,
    filter_field?: string,
    filter_value?: string,
    date_begin?: string,
    date_end?: string,
    date_enter_begin?: string,
    date_enter_end?: string,
    date_exit_begin?: string,
    date_exit_end?: string,
  }): Promise<IPage<IReportCRMHoldingParticipantLeads>>
  {
    try {
      const res = await axios.get("/api/reports/crm-holding-participant-leads", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMAggregatorLeads (params: {
    "responsible_user[]"?: Array<string>,
    filter_field?: string,
    filter_value?: string,
    date_begin?: string,
    date_end?: string,
  }): Promise<IPage<IReportCRMAggregatorLeads>>
  {
    try {
      const res = await axios.get("/api/reports/crm-aggregator-leads", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMLeadsBySalesChannels (params: {
    "responsible_user[]"?: Array<string>,
    filter_field?: string,
    filter_value?: string,
    date_begin?: string,
    date_end?: string,
  }): Promise<IPage<IReportCRMLeadsBySalesChannels>>
  {
    try {
      const res = await axios.get("/api/reports/crm-leads-by-sales-channels", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMHoldingParticipantClients (params: {
    "responsible_user[]"?: Array<string>,
    filter_field?: string,
    filter_value?: string,
    date_begin?: string,
    date_end?: string,
    date_enter_begin?: string,
    date_enter_end?: string,
    date_exit_begin?: string,
    date_exit_end?: string,
  }): Promise<IPage<IReportCRMHoldingParticipantClients>>
  {
    try {
      const res = await axios.get("/api/reports/crm-holding-participant-clients", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMAggregatorClients (params: {
    "responsible_user[]"?: Array<string>,
    filter_field?: string,
    filter_value?: string,
    date_begin?: string,
    date_end?: string,
  }): Promise<IPage<IReportCRMAggregatorClients>>
  {
    try {
      const res = await axios.get("/api/reports/crm-aggregator-clients", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMClientsBySalesChannels (params: {
    "responsible_user[]"?: Array<string>,
    filter_field?: string,
    filter_value?: string,
    date_begin?: string,
    date_end?: string,
  }): Promise<IPage<IReportCRMLeadsBySalesChannels>>
  {
    try {
      const res = await axios.get("/api/reports/crm-clients-by-sales-channels", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMDeactivatedClients (params: {
    "responsible_user[]"?: Array<string>,
    filter_field?: string,
    filter_value?: string,
    date_begin?: string,
    date_end?: string,
  }): Promise<IPage<IReportCRMDeactivatedClients>>
  {
    try {
      const res = await axios.get("/api/reports/crm-deactivated-clients", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMMerchantsStatusNew (params: {
    "responsible_user[]"?: Array<string>,
    filter_field?: string,
    filter_value?: string,
    date_begin?: string,
    date_end?: string,
  }): Promise<IPage<IReportCRMMerchantsStatusNew>>
  {
    try {
      const res = await axios.get("/api/reports/crm-merchants-status-new", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMCustomerSupportActivity (params: {
    "responsible_user[]"?: Array<string>,
    date_begin?: string,
    date_end?: string,
  }): Promise<IPage<IReportCRMCustomerSupportActivity>>
  {
    try {
      const res = await axios.get("/api/reports/crm-customer-support-activity", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCrmEmployeeGrade (params: {
    date_begin?: string,
    date_end?: string,
  }): Promise<IPage<IReportCrmEmployeeGrade>>
  {
    try {
      const res = await axios.get("/api/reports/crm-employee-grade", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getCRMCalendarTasks (params: {
    "responsible_user[]"?: Array<string>,
    status?: string,
    entity?: "Lead" | "Client",
    filter_field?: string,
    filter_value?: string,
    topic?: string,
    date_begin?: string,
    date_end?: string,
  }): Promise<IPage<IReportCRMCalendarTasks>>
  {
    try {
      const res = await axios.get("/api/reports/crm-calendar-tasks", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportTopMerchantsByIntervals (params: {
    "fiats[]"?: Array<string>;
    "types[]"?: Array<string>;
    "status[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPageReportPayportTopMerchantsByIntervals>
  {
    try {
      const res = await axios.get("/api/reports/payport-top-merchants-by-intervals", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getPayportTopMerchantsWithoutProfitability (params: {
    "status[]"?: Array<string>;
    "merchants[]"?: Array<string>;
    "sales_managers[]"?: Array<string>;
    "fiats[]"?: Array<string>;
    date_begin?: string;
    date_end?: string;
  }): Promise<IPageReportPayportTopMerchantsWithProfitability>
  {
    try {
      const res = await axios.get("/api/reports/payport-top-merchants-without-profitability", { params });
      return res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function exportReport (name: string, params: object, filename = "export.xlsx") {
    try {
      const res = await axios({
        url: "/api/reports/" + name,
        method: "GET",
        responseType: "arraybuffer",
        params: {
          export: 1,
          ...params,
        },
      });
      const url = window.URL.createObjectURL(new Blob([res.data]));
      const link = document.createElement('a');
      link.href = url;
      link.setAttribute('download', filename);
      document.body.appendChild(link);
      link.click();
      link.remove();
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  const getTSTicketInvoices = async (params: object) => {
    try {
      const res = await axios.get("/api/reports/ts-ticket-invoices", {params});

      return res.data as IPage<ITicketInvoice>
    }
    catch (e) {
      throw e
    }
  }

  const getTSUserTicketsStatuses = async (params: object) => {
    try {
      const res = await axios.get("/api/reports/ts-user-tickets-statuses", {params});

      return res.data as IPage<IReportTSUserTicketsStatus>
    }
    catch (e) {
      throw e
    }
  }

  const getTSUserTicketsByQueueStatuses = async (params: object) => {
    try {
      const res = await axios.get("/api/reports/ts-user-tickets-by-queue-statuses", {params});

      return res.data as IPage<IReportTSUserTicketsByQueueStatus>
    }
    catch (e) {
      throw e
    }
  }

  return {
    reportGroups,
    reportsBySubgroups,
    statusesList,
    typesList,
    arbitrageList,
    stepList,
    amountTypesList,
    conversionRangesList,
    dayOfWeekList,
    monthsList,
    getPayportAuthLog,
    getPayportManagerActivity,
    getPayportWithNoActivity,
    getPayportAddedToBlacklist,
    getPayportMerchantsEnteringProd,
    getPayportMerchantsWithoutTransactions,
    getPayportAverageTimeToBargain,
    getPayportTopServiceProviders,
    getPayportTopServiceProvidersWithProfitability,
    getPayportGetServiceProvidersFilter,
    getPayportTopCurrenciesWithProfitability,
    getPayportTopCurrenciesWithoutProfitability,
    getPayportGetMerchantsFilter,
    getPayportGetFiatsFilter,
    getPayportTopMerchantsWithProfitability,
    getPayportGetSalesManagersFilter,
    getPayportMerchantCurrency,
    getPayportGetTradersFilter,
    getPayportGetPaymentSystemsFilter,
    getPayportTraderConversions,
    getPayportTraderConversionsSubquery,
    getPayportMerchantConversions,
    getPayportMerchantConversionsSubquery,
    getPayportCurrencyMerchant,
    getPayportMerchantRecalculations,
    getPayportTradersSpeed,
    getPayportMerchantsEnteringSand,
    getPayportAverageTimeToProdAfterSand,
    getPayportTraderCancellations,
    getPayportTraderCancellationsSubquery,
    getPayportInvoicesDistributionByTime,
    getPayportMerchantCirculationAmountTimeline,
    getPayportFiatCirculationAmountTimeline,
    getCRMLeadsByRespAndProducts,
    getCRMConvertedCancelledLeads,
    getCRMHoldingParticipantLeads,
    getCRMAggregatorLeads,
    getCRMLeadsBySalesChannels,
    getCRMHoldingParticipantClients,
    getCRMAggregatorClients,
    getCRMClientsBySalesChannels,
    getCRMDeactivatedClients,
    getCRMMerchantsStatusNew,
    getCRMCustomerSupportActivity,
    getCrmEmployeeGrade,
    getCRMCalendarTasks,
    getPayportTopMerchantsByIntervals,
    getPayportTopMerchantsWithoutProfitability,
    exportReport,
    getTSUserTicketsStatuses,
    getTSTicketInvoices,
    getTSUserTicketsByQueueStatuses,
  };
});
