import { defineStore } from "pinia";
import axios from "@/axios";
import { AxiosError, isAxiosError } from "axios";
import {computed, ref, Ref} from "vue";
import {PaginationInterface} from "@/components/Pagination/interfaces";
import {IServiceProvider, IServiceProviderForm, IServiceProviderCurrencySettingForm, IServiceProviderCurrencySettingResponse } from "@/stores/service_providers/interfaces";
import {IFilterData, ISelect} from "@/utils/interfaces";
import { IUserShort } from "@/pages/users/interfaces";
import { IPage } from "@/utils/interfaces";
import { useI18n } from "vue-i18n";

export const useServiceProvidersStore = defineStore("service_providers", () => {
  const { t } = useI18n();

  const environmentsList: Ref<ISelect[]> = ref([
    { text: t("New"), value: "0", disabled: false },
    // { text: t("Sandbox"), value: "1", disabled: false },
    { text: t("Production"), value: "2", disabled: false },
  ]);

  const statusesList = [
    { text: t("Inactive status"), value: "0" },
    { text: t("Active status"), value: "1" },
  ] as ISelect[];

  const yesNoList = [
    { text: t("No"), value: "0" },
    { text: t("Yes"), value: "1" },
  ];

  const spTypes = [
    // { text: t("Integration"), value: "" },
    { text: t("Trader"), value: "trader" },
  ] as ISelect[];

  const directionsList = [
    { text: t("Input"), value: "0" },
    { text: t("Output"), value: "1" },
  ] as ISelect[];

  const store = ref({
    list: [] as IServiceProvider[],
    item: {} as IServiceProvider,
    meta: {} as PaginationInterface,
    listLoading: false as boolean,
    listServiceProviderCurrencySettings: {} as IServiceProviderCurrencySettingResponse,
    listServiceProviderCurrencySettingsLoading: false as boolean,
    environmentsList: environmentsList,
    statusesList: statusesList,
    yesNoList: yesNoList,
    spTypes: spTypes,
    directionsList: directionsList,
    responsibleUsersFilter: [] as Array<IUserShort>,
    fiatsIdsFilter: [] as ISelect[],
    rateSourcesFilter: [] as ISelect[],
  })

  const getList = computed(() => store.value.list)
  const getItem = computed(() => store.value.item)
  const pagination = computed(() => store.value.meta)
  const isListLoading = computed(() => store.value.listLoading)
  const getlistServiceProviderCurrencySettings = computed(() => store.value.listServiceProviderCurrencySettings)
  const isListServiceProviderCurrencySettingsLoading = computed(() => store.value.listServiceProviderCurrencySettingsLoading)
  const getFiatsIdsFilter = computed(() => store.value.fiatsIdsFilter)
  const getRateSourcesFilter = computed(() => store.value.rateSourcesFilter)

  async function list(filterData: IFilterData): Promise<IPage<IServiceProvider>> {
    try {
      store.value.list = []
      store.value.listLoading = false
      const resp = await axios.get(`api/service-providers`, {params: filterData})
      store.value.listLoading = true
      store.value.list = resp.data.data
      store.value.meta = resp.data
      return resp.data
    } catch (error: unknown) {
      if(isAxiosError(error)) {
        throw error.response
      }
      throw error
    }
  }

  async function show(id: string) {
    // if (Number(id) === store.value.item?.id) return
    try {
      const resp = await axios.get(`api/service-providers/${id}`)
      store.value.item = resp.data.data
      return resp.data.data
    } catch (error: unknown) {
      if(isAxiosError(error)) {
        throw error.response
      }
      throw error
    }
  }

  async function createServiceProvider(data: IServiceProvider | IServiceProviderForm) {
    try {
      await axios.post("/api/service-providers", data);
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateServiceProvider(id: number|string|null, data: IServiceProvider | IServiceProviderForm | { contact_persons: Array<string> }) {
    try {
      await axios.put("/api/service-providers/" + id, data);
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateServiceProviderContacts(id: number|string|null, data: { contact_persons: Array<string> }) {
    try {
      const res = await axios.put("/api/service-providers/" + id + "/contacts", data);
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function bindServiceProviderTraders(id: number|string|null, data: { traders: Array<string> }) {
    try {
      const res = await axios.put("/api/service-providers/" + id + "/bind-traders", data);
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function unbindServiceProviderTraders(id: number|string|null, data: { traders: Array<string> }) {
    try {
      const res = await axios.put("/api/service-providers/" + id + "/unbind-traders", data);
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function listServiceProviderCurrencySettings(id: number|string|null, data?: { trader_id: number }) {
    try {
      store.value.listServiceProviderCurrencySettings = {} as IServiceProviderCurrencySettingResponse
      store.value.listServiceProviderCurrencySettingsLoading = false
      const res = await axios.post("/api/service-providers/" + id + "/currency-settings/list", data);
      store.value.listServiceProviderCurrencySettingsLoading = true
      store.value.listServiceProviderCurrencySettings = res.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function saveServiceProviderCurrencySettings(id: number|string|null, data: IServiceProviderCurrencySettingForm) {
    try {
      const res = await axios.post("/api/service-providers/" + id + "/currency-settings/save", data);
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function deleteServiceProviderCurrencySettings(id: number|string|null, data: IServiceProviderCurrencySettingForm) {
    try {
      const res = await axios.post("/api/service-providers/" + id + "/currency-settings/delete", data);
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getResponsibleUsersFilter() {
    try {
      const res = await axios.get("/api/service-providers/get-responsible-users-filter");
      store.value.responsibleUsersFilter = res.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function fiatsIdsFilter ()
  {
    try {
      const resp = await axios.get(`/api/service-providers/get-fiats-ids-filter`);
      store.value.fiatsIdsFilter = resp.data.data
      return resp.data
    } catch (error: unknown) {
      if(isAxiosError(error)) {
        throw error.response
      }
      throw error
    }
  }

  async function rateSourcesFilter ()
  {
    try {
      const resp = await axios.get(`/api/service-providers/get-rate-sources-filter`);
      store.value.rateSourcesFilter = resp.data.data
      return resp.data
    } catch (error: unknown) {
      if(isAxiosError(error)) {
        throw error.response
      }
      throw error
    }
  }

  return {
    store,
    show,
    list,
    getList,
    isListLoading,
    pagination,
    getItem,
    createServiceProvider,
    updateServiceProvider,
    updateServiceProviderContacts,
    bindServiceProviderTraders,
    unbindServiceProviderTraders,
    listServiceProviderCurrencySettings,
    getlistServiceProviderCurrencySettings,
    isListServiceProviderCurrencySettingsLoading,
    saveServiceProviderCurrencySettings,
    deleteServiceProviderCurrencySettings,
    rateSourcesFilter,
    getResponsibleUsersFilter,
    getRateSourcesFilter,
    fiatsIdsFilter,
    getFiatsIdsFilter,
  };
});
