import { computed, ref } from "vue";
import { defineStore } from "pinia";
import axios from "@/axios";
import { AxiosError } from "axios";
import { IUserStatus } from "@/pages/users/interfaces";
import { IClientStatus } from "@/pages/clients/interfaces";
import {
  IProductStatus,
  ITransactionFilter,
  ICommissionsFilter,
  IMerchantBalanceFilter,
  IMerchantCryptoOperationsFilter,
  IMerchantAvailableLimitsFilter,
  IPayportAgentWithdrawalHistoryFilter,
  IPayportAgentMerchantsFilter,
  PayportMerhcantStatuses,
  PayportAgentProductStatuses,
  IClientTariffsFilter,
  IClientInvoicesFilter,
  GnuVpnClientProductStatuses,
} from "@/pages/products/interfaces";
import { ILeadStatus, ILeadEditableStatus } from "@/pages/leads/interfaces";
import { ICommunicationStatus } from "./communication/interfaces";
import {IFilterData} from "@/utils/interfaces";
import {IFinancialIndicatorsFilter} from "@/pages/holding/interfaces";
import {LocationQueryValue} from "vue-router";
import { IContactStatus } from "@/stores/contact_person/interfaces";
import { useI18n } from "vue-i18n";

export const useStatusStore = defineStore("status", () => {
  const {t} = useI18n();

  const yesNoFilter = [
    { text: t("No"), value: "0" },
    { text: t("Yes"), value: "1" },
  ];

  const store = ref({
    userStatuses: {} as IUserStatus,
    clientStatuses: {} as IClientStatus,
    leadStatuses: {} as ILeadStatus,
    leadEditableStatuses: {} as ILeadEditableStatus,
    productStatuses: {} as IProductStatus,
    payportMerchantProductStatuses: {} as PayportMerhcantStatuses,
    payportMerchantStatuses: {} as {[key: number]: string },
    paymentInvoiceCheckList: {} as {[key: number]: string },
    payportAgentProductStatuses: {} as PayportAgentProductStatuses,
    gnuVpnClientProductStatuses: {} as GnuVpnClientProductStatuses,
    payportTransactionFilter: {} as ITransactionFilter,
    holdingTransactionFilter: {} as ITransactionFilter,
    holdingFinancialIndicatorsFilter: {} as IFinancialIndicatorsFilter,
    payportAgentTransactionFilter: {} as ITransactionFilter,
    payportCommissionsFilter: {} as ICommissionsFilter,
    clientTariffsFilter: {} as IClientTariffsFilter,
    clientInvoicesFilter: {} as IClientInvoicesFilter,
    payportMerchantBalanceFilter: {} as IMerchantBalanceFilter,
    communicationStatuses: [] as Array<{
      id: string,
      name: string,
      name_original: string,
      statuses: ICommunicationStatus,
    }>,
    payportMerchantCryptoOperationsFilter: {} as IMerchantCryptoOperationsFilter,
    payportMerchantAvailableLimits: {} as IMerchantAvailableLimitsFilter,
    payportAgentWithdrawalHistory: {} as IPayportAgentWithdrawalHistoryFilter,
    payportAgentMerchants: {} as IPayportAgentMerchantsFilter,
    actualProductStatus: '' as string,
    contactStatuses: {} as IContactStatus,
    taskStatuses: {} as { [key: string] : string },
  })

  const getUserStatuses = computed(() => store.value.userStatuses);
  const getClientStatuses = computed(() => store.value.clientStatuses);
  const getLeadStatuses = computed(() => store.value.leadStatuses);
  const getLeadStatusesEditable = computed(() => store.value.leadEditableStatuses);
  const getProductStatuses = computed(() => store.value.productStatuses);
  const getClientTariffsFilter = computed(() => store.value.clientTariffsFilter);
  const getClientInvoicesFilter = computed(() => store.value.clientInvoicesFilter);
  const getPayportMerchantProductStatuses = computed(() => store.value.payportMerchantProductStatuses);
  const getPayportMerchantStatuses = computed(() => store.value.payportMerchantStatuses);
  const getPaymentInvoiceCheckList = computed(() => store.value.paymentInvoiceCheckList);
  const getPayportAgentProductStatuses = computed(() => store.value.payportAgentProductStatuses);
  const getGnuVpnClientProductStatuses = computed(() => store.value.gnuVpnClientProductStatuses);
  const getPayportCommissionsFilter = computed(() => store.value.payportCommissionsFilter);
  const getPayportTransactionFilter = computed(() => store.value.payportTransactionFilter);
  const getHoldingTransactionFilter = computed(() => store.value.holdingTransactionFilter);
  const getHoldingFinancialIndicatorsFilter = computed(() => store.value.holdingFinancialIndicatorsFilter);
  const getPayportAgentTransactionFilter = computed(() => store.value.payportAgentTransactionFilter);
  const getPayportMerchantBalanceFilter = computed(() => store.value.payportMerchantBalanceFilter);
  const getCommunicationStatuses = computed(() => store.value.communicationStatuses);
  const getPayportMerchantCryptoOperationsFilter = computed(() => store.value.payportMerchantCryptoOperationsFilter);
  const getPayportMerchantAvailableLimits = computed(() => store.value.payportMerchantAvailableLimits)
  const getPayportAgentWithdrawalHistory = computed(() => store.value.payportAgentWithdrawalHistory)
  const getPayportAgentMerchantsFilter = computed(() => store.value.payportAgentMerchants)
  const getPayportActualProductStatus = computed(() => store.value.actualProductStatus)
  const getContactStatuses = computed(() => store.value.contactStatuses);
  const getTaskStatuses = computed(() => store.value.taskStatuses);

  async function getAllUserStatuses() {
    try {
      const res = await axios.get("/api/users/statuses");
      store.value.userStatuses = res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getAllClientStatuses() {
    try {
      const res = await axios.get("/api/clients/statuses");
      store.value.clientStatuses = res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getAllLeadStatuses() {
    try {
      const res = await axios.get("/api/leads/statuses");
      store.value.leadStatuses = res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response
    }
  }

  async function getEditableLeadStatuses() {
    try {
      const res = await axios.get("/api/leads/statuses", {
        params: {
          editable: 1
        }
      });
      store.value.leadEditableStatuses = res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response
    }
  }

  async function getAllProductStatuses() {
    try {
      const res = await axios.get("/api/products/statuses");
      store.value.productStatuses = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getActualProductStatus(product_id: string|number) {
    try {
      const res = await axios.get("/api/products/get_actual_product_status",{
        params:{
          product_id: product_id
        }
      });
      store.value.actualProductStatus = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllPayportMerchantProductStatuses() {
    try {
      const res = await axios.get("/api/products/payport_merchant_statuses");
      store.value.payportMerchantProductStatuses = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }
  async function getAllPayportMerchantStatuses() {
    try {
      const res = await axios.get("/api/traders/merchants/list");
      store.value.payportMerchantStatuses = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllPayportAgentProductStatuses() {
    try {
      const res = await axios.get("/api/products/payport_agent_statuses");
      store.value.payportAgentProductStatuses = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllGnuVpnClientProductStatuses() {
    try {
      const res = await axios.get("/api/products/gnu_vpn_client_statuses");
      store.value.gnuVpnClientProductStatuses = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllPayportTransactionFilter() {
    try {
      const res = await axios.get("/api/products/get_transactions_filter");
      store.value.payportTransactionFilter = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }
  async function getAllHoldingTransactionFilter(params: IFilterData = {}) {
    try {
      const res = await axios.get("/api/clients/get_holding_transactions_filter", {params});
      store.value.holdingTransactionFilter = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }
  async function getAllHoldingFinancialIndicatorsFilter(params: IFilterData = {}) {
    try {
      const res = await axios.get("/api/clients/get_holding_financial_indicators_filter", {params});
      store.value.holdingFinancialIndicatorsFilter = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }
  async function getAllPayportAgentTransactionFilter(agent_id:string|number) {
    try {
      const res = await axios.get("/api/products/get_agent_transactions_filter",{
        params:{
          agent_id: agent_id
        }
      });
      store.value.payportAgentTransactionFilter = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllPayportCommissionsFilter(merchant_id:string|number) {
    try {
      const res = await axios.get("/api/products/get_commissions_filter",{
        params: {
          merchant_id: merchant_id
        }
      });
      store.value.payportCommissionsFilter = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllPayportMerchantBalanceFilter() {
    try {
      const res = await axios.get("/api/products/get_merchant_balance_filter");
      store.value.payportMerchantBalanceFilter = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllCommunicationStatuses() {
    try {
      const res = await axios.get("/api/communications/statuses");
      store.value.communicationStatuses = res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getAllPayportMerchantCryptoOperationsFilter(merchant_id:string|number) {
    try {
      const res = await axios.get("/api/products/get_merchant_crypto_operations_filter",{
        params: {
          merchant_id: merchant_id
        }
      });
      store.value.payportMerchantCryptoOperationsFilter = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }
  async function getAllPayportMerchantAvailableLimits() {
    try {
      const res = await axios.get("/api/products/get_merchant_available_limits_filter");
      store.value.payportMerchantAvailableLimits = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllPayportAgentWithdrawalHistory(agent_id:string|number) {
    try {
      const res = await axios.get("/api/products/get_agent_withdrawal_history_filter",{
        params:{
          agent_id: agent_id
        }
      });
      store.value.payportAgentWithdrawalHistory = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllPayportAgentMerchantsFilter() {
    try {
      const res = await axios.get("/api/products/get_agent_merchants_filter");
      store.value.payportAgentMerchants = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }
  async function getAllPaymentInvoiceCheckList() {
    try {
      const res = await axios.get("/api/payment-invoice/check/list");
      store.value.paymentInvoiceCheckList = res.data.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllClientTariffsFilter() {
    try {
      const res = await axios.get("/api/products/get_client_tariffs_filter");
      store.value.clientTariffsFilter = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllClientInvoicesFilter(email:string) {
    try {
      const res = await axios.get("/api/products/get_client_invoices_filter", {
        params: {
          email: email
        }
      });
      store.value.clientInvoicesFilter = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function getAllContactStatuses() {
    try {
      const res = await axios.get("/api/contact-persons/statuses");
      store.value.contactStatuses = res.data;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  async function getAllTaskStatuses() {
    try {
      const res = await axios.get(
        "/api/communications/statuses",
        { params: { type: "Task" }}
      );
      store.value.taskStatuses = res.data[0].statuses;
    } catch (err) {
      const error = err as AxiosError;
      throw error.response;
    }
  }

  return {
    yesNoFilter,
    getUserStatuses,
    getAllUserStatuses,
    getClientStatuses,
    getAllClientStatuses,
    getLeadStatuses,
    getEditableLeadStatuses,
    getLeadStatusesEditable,
    getAllLeadStatuses,
    getProductStatuses,
    getAllProductStatuses,
    getPayportMerchantProductStatuses,
    getPayportAgentProductStatuses,
    getGnuVpnClientProductStatuses,
    getAllPayportMerchantProductStatuses,
    getAllPayportAgentProductStatuses,
    getAllGnuVpnClientProductStatuses,
    getPayportTransactionFilter,
    getAllPayportTransactionFilter,
    getPayportAgentTransactionFilter,
    getAllHoldingTransactionFilter,
    getHoldingTransactionFilter,
    getAllHoldingFinancialIndicatorsFilter,
    getHoldingFinancialIndicatorsFilter,
    getAllPayportAgentTransactionFilter,
    getPayportCommissionsFilter,
    getAllPayportCommissionsFilter,
    getPayportMerchantBalanceFilter,
    getAllPayportMerchantBalanceFilter,
    getCommunicationStatuses,
    getAllCommunicationStatuses,
    getPayportMerchantCryptoOperationsFilter,
    getAllPayportMerchantCryptoOperationsFilter,
    getPayportMerchantAvailableLimits,
    getAllPayportMerchantAvailableLimits,
    getPayportAgentWithdrawalHistory,
    getAllPayportAgentWithdrawalHistory,
    getPayportAgentMerchantsFilter,
    getAllPayportAgentMerchantsFilter,
    getActualProductStatus,
    getPayportActualProductStatus,
    getAllPayportMerchantStatuses,
    getPayportMerchantStatuses,
    getAllPaymentInvoiceCheckList,
    getPaymentInvoiceCheckList,
    getClientTariffsFilter,
    getAllClientTariffsFilter,
    getClientInvoicesFilter,
    getAllClientInvoicesFilter,
    getAllContactStatuses,
    getContactStatuses,
    getAllTaskStatuses,
    getTaskStatuses,
  };
})
