import { defineStore } from "pinia";
import axios from "@/axios";
import { AxiosError, isAxiosError } from "axios";
import {computed, ref, Ref} from "vue";
import {PaginationInterface} from "@/components/Pagination/interfaces";
import {ITrader, ITraderForm} from "@/stores/trader/interfaces";
import { IServiceProviderCurrencySettingForm } from "@/stores/service_providers/interfaces";
import {IFilterData, ISelect} from "@/utils/interfaces";
import { IUserShort } from "@/pages/users/interfaces";
import { IPage } from "@/utils/interfaces";
import { useI18n } from "vue-i18n";

export const useTraderStore = defineStore("trader", () => {
  const { t } = useI18n();

  const environmentsList: Ref<ISelect[]> = ref([
    { text: t("New"), value: "0", disabled: false },
    // { text: t("Sandbox"), value: "1", disabled: false },
    { text: t("Production"), value: "2", disabled: false },
  ]);

  const statusesList = [
    { text: t("Inactive status"), value: "0" },
    { text: t("Active status"), value: "1" },
  ];

  const yesNoList = [
    { text: t("No"), value: "0" },
    { text: t("Yes"), value: "1" },
  ];

  const languagesList = [
    { text: t("Russian"), value: "ru" },
    { text: t("English"), value: "en" },
  ];

  const store = ref({
    list: [] as ITrader[],
    item: {} as ITrader,
    meta: {} as PaginationInterface,
    agentsFilter: [] as ISelect[],
    environmentsList: environmentsList,
    statusesList: statusesList,
    yesNoList: yesNoList,
    languagesList: languagesList,
    selectList: [] as ISelect[],
    responsibleUsersFilter: [] as Array<IUserShort>,
    listServiceProviderCurrencySettingIds: [] as Array<number>,
    updateSpCurrencySettingsLoading: false as boolean,
  })

  const getList = computed(() => store.value.list)
  const getSelectList = computed(() => store.value.selectList)
  const getItem = computed(() => store.value.item)
  const pagination = computed(() => store.value.meta)
  const getServiceProviderCurrencySettingIds = computed(() => store.value.listServiceProviderCurrencySettingIds)

  async function list(filterData: IFilterData): Promise<IPage<ITrader>> {
    try {
      store.value.list = []
      const resp = await axios.get(`api/traders`, {params: filterData})
      store.value.list = resp.data.data
      store.value.meta = resp.data
      return resp.data
    } catch (error: unknown) {
      if(isAxiosError(error)) {
        throw error.response
      }
      throw error
    }
  }

  async function listServiceProviderCurrencySettingIds(id: number|string|null) {
    try {
      store.value.listServiceProviderCurrencySettingIds = []
      const res = await axios.get("/api/trader/" + id + "/service-provider-currency-settings-ids");
      store.value.listServiceProviderCurrencySettingIds = res.data.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateServiceProviderCurrencySettingIds(id: number|string|null, data: { settings_ids: Array<string> }) {
    try {
      store.value.listServiceProviderCurrencySettingIds = []
      const res = await axios.post("/api/trader/" + id + "/service-provider-currency-settings-ids", data);
      store.value.listServiceProviderCurrencySettingIds = res.data.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function show(id: string) {
    // if (Number(id) === store.value.item?.id) return
    try {
      const resp = await axios.get(`api/traders/${id}`)
      store.value.item = resp.data.data
      return resp.data.data
    } catch (error: unknown) {
      if(isAxiosError(error)) {
        throw error.response
      }
      throw error
    }
  }

  async function getAgentsFilter() {
    try {
      const resp = await axios.get("/api/agents/list");
      store.value.agentsFilter = resp.data.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function createTrader(data: ITrader | ITraderForm) {
    try {
      await axios.post("/api/traders", data);
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateTrader(id: number|string|null, data: ITrader | ITraderForm) {
    try {
      await axios.put("/api/traders/" + id, data);
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateSpCurrencySettings(id: number|string|null,data: { sp_currency_settings_ids: Array<string>, old_sp_currency_settings: Array<IServiceProviderCurrencySettingForm> }) {
    try {
      store.value.updateSpCurrencySettingsLoading = true
      await axios.put("/api/traders/" + id + "/update-sp-currency-settings", data);
      store.value.updateSpCurrencySettingsLoading = false
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateTraderContacts(id: number|string|null, data: { contact_persons: Array<string> }) {
    try {
      const res = await axios.put("/api/traders/" + id + "/contacts", data);
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getResponsibleUsersFilter() {
    try {
      const res = await axios.get("/api/traders/get-responsible-users-filter");
      store.value.responsibleUsersFilter = res.data
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function selectList() {
    try {
      const resp = await axios.get('/api/traders/list')
      store.value.selectList = resp.data.data
    } catch (e) {
      throw e
    }
  }

  return { store, show, list, getList, pagination, getItem, getAgentsFilter, createTrader, updateTrader, updateSpCurrencySettings, updateTraderContacts, selectList, getSelectList, getResponsibleUsersFilter, listServiceProviderCurrencySettingIds, getServiceProviderCurrencySettingIds, updateServiceProviderCurrencySettingIds };
});
