import {defineStore} from "pinia";
import axios from "@/axios";
import {computed, ref} from "vue";
import {
    InvoiceHistory,
    IDealHistory,
    InvoiceTypes,
    ISearchableTransaction,
    ITransaction,
    ITransactionFilter,
    ITransactionShow,
    ITransactionTotal,
} from "@/stores/transaction/interfaces";
import {IPage} from "@/utils/interfaces";

export const useTransactionStore = defineStore("transaction", () => {
  const store = ref({
    total: {} as ITransactionTotal,
    page: {} as IPage<ITransaction>,
    filters: null as ITransactionFilter | null,

    transaction: {} as ITransactionShow,
    selectableTransactionsById: [] as ISearchableTransaction[],
    selectableTransactionsByPaymentId: [] as ISearchableTransaction[],
    invoiceHistory: [] as InvoiceHistory[],
    dealHistory: [] as IDealHistory[],
    statusNames: {
      0: "Created",
      1: "Paid",
      2: "Confirmed",
      3: "Trader accepted",
      4: "In process",
      5: "User selected",
      60: "EcomPayed",
      96: "Hold",
      97: "Canceled by revert",
      98: "Cancelled by timeout",
      99: "Cancelled"
    } as {
        [key: string]: string; // Allows any string key
    },
    traderDealStatus: {
      0: "Created",
      1: "Complete",
      2: "Trader funded",
      3: "Apply user",
      4: "Apply trader",
      5: "Arbitrage",
      6: "Crypto Store Wait",
      94: "Cancel by Reply Timeout",
      95: "Cancel by Timeout",
      96: "Cancel by SD",
      97: "Cancel by Trader",
      98: "Cancel by User",
      99: "Canceled"
    } as {
        [key: string]: string; // Allows any string key
    },
    nettingDealStatus: {
      0: "Created",
      1: "Completed",
      2: "Trader Accept Deal",
      5: "Pending",
      6: "Preprocessing",
      7: "Waiting For Accept",
      94: "Cancel by Error",
      95: "Cancel by Timeout",
      96: "Cancel by SD",
      97: "Cancel by Trader",
      98: "Cancel by User",
      99: "Cancel"
    } as {
          [key: string]: string; // Allows any string key
      },
    cryptoDealStatus: {
      0: "Created",
      1: "Paid",
      2: "Confirmed",
      99: "Canceled"
    } as {
          [key: string]: string; // Allows any string key
      },
    cryptoCancellationReasons: {
      0: "CANCELED_BY_USER",
      1: "CANCELED_BY_EXPIRED",
      3: "CANCELED_BY_MERCHANT",
      4: "CANCELED_BY_SD"
    } as {
          [key: string]: string; // Allows any string key
      }
  })

  const filters = computed(() => store.value.filters)
  const transactions = computed(() => store.value.page.data)
  const total = computed(() => store.value.total)
  const pagination = computed(() => store.value.page)

  const getSelectableInvoicesById = computed(() => store.value.selectableTransactionsById)
  const getSelectableInvoicesByPaymentId = computed(() => store.value.selectableTransactionsByPaymentId)
  const getTransaction = computed(() => store.value.transaction)
  const getInvoiceHistory = computed(() => store.value.invoiceHistory)
  const getDealHistory = computed(() => store.value.dealHistory)

  async function fetchTransactions(query?: object) {
    try {
      const res = await axios.get("/api/transactions", {
        params: query,
      })

      store.value.page = res.data.pagination
      store.value.total = res.data.transaction_total

    } catch (error: any) {
      throw error.response
    }
  }

  async function fetchAllFilters() {
    try {
      const res = await axios.get("/api/transactions/filters-extended");
      store.value.filters = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function fetchFilters() {
    try {
      const res = await axios.get("/api/transactions/filters");
      store.value.filters = res.data;
    } catch(error: any) {
      throw error.response;
    }
  }

  async function show(type: string, id: string) {
    try {
      const res = await axios.get('/api/transactions/show', {
        params: {id, type}
      })

      store.value.selectableTransactionsById = res.data.data;
    }
    catch (e) {
      throw e
    }
  }

  async function fetchInvoiceHistory(type: string, id: string) {
    try {
      const res = await axios.get('/api/transactions/invoice_history', {
        params: {id, type}
      })

      store.value.invoiceHistory = res.data.data;
    }
    catch (e) {
      throw e
    }
  }

  async function fetchDealHistory(type: string, id: string) {
    try {
      const res = await axios.get('/api/transactions/deal_history', {
        params: {id, type}
      })

      store.value.dealHistory = res.data.data;
    }
    catch (e) {
      throw e
    }
  }
  async function fetchSearchTransactionsById(id: string) {
    try {
      const res = await axios.get('/api/transactions/search-select', {
        params: {id}
      })

      store.value.selectableTransactionsById = res.data.data;
    }
    catch (e) {
      throw e
    }
  }

  async function fetchTransactionsByPaymentId(paymentId: string) {
    try {
      const res = await axios.get('/api/transactions/search-select', {
        params: {
          payment_id: paymentId,
        }
      })

      store.value.selectableTransactionsByPaymentId = res.data.data
    }
    catch (e) {
      throw e
    }
  }

  async function fetchTransaction(transactionId: string, invoiceTypes: InvoiceTypes) {
    try {
      const res = await axios.get('/api/transactions/show', {
        params: {
          id: transactionId,
          type: invoiceTypes,
        }
      })

      store.value.transaction = res.data.data
    }
    catch (e) {
      throw e
    }
  }

  function clearSelect() {
    store.value.selectableTransactionsById = []
    store.value.selectableTransactionsByPaymentId = []
  }

  return {
    store,

    fetchFilters,
    fetchAllFilters,
    fetchTransactions,
    fetchSearchTransactionsById,
    fetchTransactionsByPaymentId,
    fetchTransaction,
    fetchInvoiceHistory,
    fetchDealHistory,

    getSelectableInvoicesById,
    getSelectableInvoicesByPaymentId,
    getTransaction,
    getInvoiceHistory,
    getDealHistory,

    clearSelect,

    pagination,
    filters,
    transactions,
    total,
    show
  }
})
