import {IFilterData, IPage, ISelect} from "@/utils/interfaces";
import {
  IGetTreeSelectListParams,
  IQueue,
  IQueueForList,
  IQueueShort,
  IQueueUpdateForm
} from "@/pages/ts/queues/interfaces";
import {Queue} from "@/pages/ts/queues/classes";
import {defineStore} from "pinia";
import axios from "@/axios";
import {AxiosError} from "axios";
import {TreeNode} from "primevue/tree";

const URL = "/api/ts/queues";

export const useQueueStore = defineStore("queues", () => {
  async function getAllRows(): Promise<Array<IQueue>> {
    try {
      const res = await axios.get(URL + "/all");
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getRows(params: IFilterData = {}): Promise<IPage<IQueueForList>> {
    try {
      const res = await axios.get(URL, {params});
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getTreeSelectList(params?: IGetTreeSelectListParams): Promise<TreeNode[]> {
    try {
      const res = await axios.get(URL + '/tree-select', {
        params: {
          status: params?.status ?? null,
          active_by_tickets: params?.active_by_tickets ? Number(params.active_by_tickets) : null,
          exclude_self_id: params?.exclude_self_id ?? null,
          skip_role_check: params?.skip_role_check ? Number(params.skip_role_check) : null,
          by_role: params?.by_role ?? null,
          user_id_can_create: params?.user_id_can_create ?? null,
          role_can_create: params?.role_can_create ?? null,
        } as IGetTreeSelectListParams
      });
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getRowsShort(params: IFilterData = {}): Promise<Array<IQueueShort>> {
    try {
      const res = await axios.get(URL + '/short', {params});
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getOneRow(id: number | string | null): Promise<IQueue> {
    try {
      const res = await axios.get(URL + "/" + id + "/edit");
      return new Queue(res.data);
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function createRow(params: IQueueUpdateForm): Promise<IQueue> {
    try {
      const res = await axios.post(URL, params);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateRow(id: number | string | null, data: IQueueUpdateForm): Promise<IQueue> {
    try {
      const res = await axios.put(URL + "/" + id, data);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getOwnerSelectList(): Promise<ISelect[]> {
    try {
      const res = await axios.get(URL + "/owner_select_list");
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getExecutorSelectList(queueId: number | string): Promise<ISelect[]> {
    try {
      const res = await axios.get(URL + "/executor_select_list/" + queueId);

      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getPermittedUserSelectList(queueId?: number | string | null): Promise<ISelect[]> {
    try {
      const res = await axios.get(URL + '/permitted-users', {
        params: {
          queue_id: queueId,
        }
      });

      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  return {
    getAllRows,
    getRows,
    getRowsShort,
    getOneRow,
    createRow,
    updateRow,
    getTreeSelectList,
    getOwnerSelectList,
    getExecutorSelectList,
    getPermittedUserSelectList,
  };
});
