import {IFilterData, IPage} from "@/utils/interfaces";
import {
  ITicket,
  ITicketForList,
  ITicketForMassActions,
  ITicketUpdateForm,
  ITicketUpdatePartForm,
} from "@/pages/ts/tickets/interfaces";
import {Ticket} from "@/pages/ts/tickets/classes";
import {defineStore} from "pinia";
import axios from "@/axios";
import {AxiosError} from "axios";

const URL = "/api/ts/tickets";

export const useTicketStore = defineStore("tickets", () => {
  async function getAllRows(): Promise<Array<ITicket>> {
    try {
      const res = await axios.get(URL + "/all");
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getRows(params: IFilterData = {}, headers?: { [key: string]: string }): Promise<IPage<ITicketForList>> {
    try {
      const res = await axios.get(URL, {params, headers});
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getRowsFullData(params: IFilterData = {}): Promise<IPage<ITicket>> {
    try {
      params.with_full_data = 1;
      const res = await axios.get(URL, {params});
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getRowsForMassActions(params: IFilterData = {}): Promise<IPage<ITicketForMassActions>> {
    try {
      params.for_mass_actions = 1;
      const res = await axios.get(URL, {params});
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getTicketEmails(params: IFilterData = {}): Promise<Array<string>> {
    try {
      const res = await axios.get(URL + "/customer_emails", {params});
      return res.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function getOneRow(id: number | string | null): Promise<ITicket> {
    try {
      const res = await axios.get(URL + "/" + id + "/edit");
      return new Ticket(res.data);
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function createRow(params: ITicketUpdateForm): Promise<ITicket> {
    try {
      const res = await axios.post(URL, params);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateRow(id: number | string | null, data: ITicketUpdatePartForm): Promise<ITicket> {
    try {
      const res = await axios.put(URL + "/" + id, data);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  async function updateRows(data: ITicketUpdatePartForm) {
    try {
      const res = await axios.put(URL + "/mass", data);
      return res.data.data;
    } catch (error) {
      const err = error as AxiosError;
      throw err.response;
    }
  }

  return {getAllRows, getRows, getRowsFullData, getRowsForMassActions, getTicketEmails, getOneRow, createRow, updateRow, updateRows};
});
