import { defineStore } from 'pinia'
import axios from '@/axios'
import { AxiosError } from "axios";
import { ref, watch } from 'vue'
import {ILoginForm, IPasswordResetForm} from "@/stores/user/interfaces";

import { usePusher } from "@/pusher";

const { subscribe, unsubscribe } = usePusher();
import { IUpdateUserForm } from '@/pages/users/interfaces';
import { IDepartment } from './department/interfaces';

export const useUserStore = defineStore('user', () => {

  const auth = ref({
    loggedIn: false,
    user: {} as User,
    selectList: [],
  } as Auth);

  watch(() => auth.value.user, (newValue) => {
    if (Object.keys(newValue).includes("id")) {
      subscribe({
        user_id: newValue.id,
        role_id: newValue.roles.map(role => role.id),
        department_id: newValue.departments.map(department => department.id),
      });
    }
  });

    interface Auth {
      loggedIn: boolean,
      user: User,
      selectList: []
    }

    interface User {
      readonly id: string
      name: string,
      full_name: string,
      roles: Role[],
      departments: IDepartment[],
      disabled_web_notifications: number[],
      time_zone: string,
      permissions: {
        [key: string]: {
          [key: string]: boolean
        }
      },
      communication_display_level: "own_and_departments" | "all" | "",
      forbidden_admin_permissions: string[],
    }

    interface Role {
      id:number;
      name: string;
      permissions: RolePermission[];
    }

    interface RolePermission {
      name: string;
    }

    function isLoggedIn() {
      return auth.value.loggedIn
    }

    function setLoggedIn(payload: boolean) {
      auth.value.loggedIn = payload
    }

    async function logout() {
      try {
        unsubscribe();
        const resp = await axios.post(`api/forty/logout`)
        auth.value.loggedIn = false
        auth.value.user = {} as User
        localStorage.removeItem("hasNewNotification");
        localStorage.removeItem("soundOn");
        return resp.data
      } catch (error: any) {
        throw error.response
      }
    }

    function setUser(payload: User) {
      auth.value.user = payload
    }

    async function login (payload: ILoginForm) {
      try {
        await axios.get('api/sanctum/csrf-cookie');
        const user = useUserStore()
        const resp = await axios.post(`api/forty/login`, payload)
        await user.loadUserData()
        return resp
      } catch (error: any) {
        throw error.response
      }
    }

    async function loadUserData () {
      const resp = await axios.get(`api/user`);

      setLoggedIn(true);
      setUser(resp.data);
      return resp;
    }

    async function getUser () {
      try {
        const user = useUserStore()
        if(user.isLoggedIn()) {
          return  auth.value.user
        }
        const resp = await loadUserData();
        return resp.data;
      } catch (error: any) {
        throw error.response
      }
    }

    async function getUserSelectList () {
      try {
        const user = useUserStore()
        const resp = await axios.get(`api/user/list`)
        auth.value.selectList = resp.data
        return resp.data
      } catch (error: any) {
        throw error.response
      }
    }
    async function restore (payload: IPasswordResetForm) {
      try {
        const resp = await axios.post(`api/forty/forgot-password`, payload)
      } catch (error: any) {
        throw error.response
      }
    }

    function isUserHasRole (payload: string) {
      if(auth.value.user.roles) {
        return Boolean(auth.value.user.roles.find((item: Role) => item.name === payload))
      }
      return false
    }

    type IPermission = boolean|Array<number>|unknown
    function isUserHasPermission (table: string, action: string, id?: string | number, onlyAll?: boolean): IPermission {
      if (!auth.value.user.hasOwnProperty("permissions") || !auth.value.user.permissions.hasOwnProperty(table))
        return false;

      const permission: IPermission = auth.value.user.permissions[table][action];

      if (onlyAll) return permission === true;

      if (!!id && Array.isArray(permission)) return permission.includes(id);  // own or own departments

      let result = permission;

      if ((typeof permission == "boolean" && ["index", "create"].includes(action))
          ||
          (table == "contact_persons" && action == "show"))
      {
        const ownName = `${table}_${action}_own`;
        const ownDepartmentName = `${table}_${action}_own_and_departments`;

        for (const role of auth.value.user.roles) {
          if (role.hasOwnProperty("permissions")
              && !!role.permissions.find(item => {
                if (role.name == 'admin' && auth.value.user.forbidden_admin_permissions.includes(table)) {
                  return false;
                }
                return [ownName, ownDepartmentName].includes(item.name)
              }))
          {
            result = true;
            break;
          }
        }
      }

      return result;
    }

    async function updateUser (userParams: IUpdateUserForm) {
      try {
        await axios.put("/api/user", userParams);
      } catch (error) {
        const err = error as AxiosError;
        throw err.response;
      }
    }

    return {
      isLoggedIn,
      login,
      auth,
      logout,
      setLoggedIn,
      getUser,
      setUser,
      isUserHasRole,
      getUserSelectList,
      isUserHasPermission,
      restore,
      loadUserData,
      updateUser,
    }
})
