import dayjs from "dayjs";
import duration from "dayjs/plugin/duration";
import timezone from "dayjs/plugin/timezone";
import utc from 'dayjs/plugin/utc'

import { parseColor } from "tailwindcss/lib/util/color";
import { ErrorObject } from "@vuelidate/core";
import customParseFormat from "dayjs/plugin/customParseFormat";
import quarterOfYear from "dayjs/plugin/quarterOfYear";
import {IFile} from "@/stores/file/interfaces";

dayjs.extend(utc)
dayjs.extend(timezone);
dayjs.extend(duration);
dayjs.extend(customParseFormat);
dayjs.extend(quarterOfYear);

const getNewMetaEnv = () => {
  return Object.assign({}, import.meta.env, {
    'VITE_APP_API_URL': import.meta.env.VITE_APP_API_URL !== undefined && import.meta.env.VITE_APP_API_URL ? import.meta.env.VITE_APP_API_URL : (window.location.protocol + '//' + window.location.hostname),
    'VITE_PUSHER_WS_HOST': import.meta.env.VITE_PUSHER_WS_HOST !== undefined && import.meta.env.VITE_PUSHER_WS_HOST ? import.meta.env.VITE_PUSHER_WS_HOST : window.location.host,
    'VITE_SENTRY_DSN': import.meta.env.VITE_SENTRY_DSN !== undefined && import.meta.env.VITE_SENTRY_DSN ? import.meta.env.VITE_SENTRY_DSN : '',
  })
}

const metaEnv = getNewMetaEnv()

const cutText = (text: string, length: number) => {
  if (text.split(" ").length > 1) {
    const string = text.substring(0, length);
    const splitText = string.split(" ");
    splitText.pop();
    return splitText.join(" ") + "...";
  } else {
    return text;
  }
};

const formatDate = (date: string, format: string) => {
  return dayjs(date).format(format);
};

const capitalizeFirstLetter = (string: string) => {
  if (string) {
    return string.charAt(0).toUpperCase() + string.slice(1);
  } else {
    return "";
  }
};

const onlyNumber = (string: string) => {
  if (string) {
    return string.replace(/\D/g, "");
  } else {
    return "";
  }
};

const formatCurrency = (number: number) => {
  if (number) {
    const formattedNumber = number.toString().replace(/\D/g, "");
    const rest = formattedNumber.length % 3;
    let currency = formattedNumber.substr(0, rest);
    const thousand = formattedNumber.substr(rest).match(/\d{3}/g);
    let separator;

    if (thousand) {
      separator = rest ? "." : "";
      currency += separator + thousand.join(".");
    }

    return currency;
  } else {
    return "";
  }
};

const timeAgo = (time: string) => {
  const date = new Date((time || "").replace(/-/g, "/").replace(/[TZ]/g, " "));
  const diff = (new Date().getTime() - date.getTime()) / 1000;
  const dayDiff = Math.floor(diff / 86400);

  if (isNaN(dayDiff) || dayDiff < 0 || dayDiff >= 31) {
    return dayjs(time).format("MMMM DD, YYYY");
  }

  return (
    (dayDiff === 0 &&
      ((diff < 60 && "just now") ||
        (diff < 120 && "1 minute ago") ||
        (diff < 3600 && Math.floor(diff / 60) + " minutes ago") ||
        (diff < 7200 && "1 hour ago") ||
        (diff < 86400 && Math.floor(diff / 3600) + " hours ago"))) ||
    (dayDiff === 1 && "Yesterday") ||
    (dayDiff < 7 && dayDiff + " days ago") ||
    (dayDiff < 31 && Math.ceil(dayDiff / 7) + " weeks ago")
  );
};

const diffTimeByNow = (time: string) => {
  const startDate = dayjs(dayjs().format("YYYY-MM-DD HH:mm:ss").toString());
  const endDate = dayjs(dayjs(time).format("YYYY-MM-DD HH:mm:ss").toString());

  const duration = dayjs.duration(endDate.diff(startDate));
  const milliseconds = Math.floor(duration.asMilliseconds());

  const days = Math.round(milliseconds / 86400000);
  const hours = Math.round((milliseconds % 86400000) / 3600000);
  let minutes = Math.round(((milliseconds % 86400000) % 3600000) / 60000);
  const seconds = Math.round(
    (((milliseconds % 86400000) % 3600000) % 60000) / 1000
  );

  if (seconds < 30 && seconds >= 0) {
    minutes += 1;
  }

  return {
    days: days.toString().length < 2 ? "0" + days : days,
    hours: hours.toString().length < 2 ? "0" + hours : hours,
    minutes: minutes.toString().length < 2 ? "0" + minutes : minutes,
    seconds: seconds.toString().length < 2 ? "0" + seconds : seconds,
  };
};

const isset = (obj: object | string) => {
  if (obj !== null && obj !== undefined) {
    if (typeof obj === "object" || Array.isArray(obj)) {
      return Object.keys(obj).length;
    } else {
      return obj.toString().length;
    }
  }

  return false;
};

const toRaw = (obj: object) => {
  return JSON.parse(JSON.stringify(obj));
};

const randomNumbers = (from: number, to: number, length: number) => {
  const numbers = [0];
  for (let i = 1; i < length; i++) {
    numbers.push(Math.ceil(Math.random() * (from - to) + to));
  }

  return numbers;
};

const toRGB = (value: string) => {
  return parseColor(value).color.join(" ");
};

const stringToHTML = (arg: string) => {
  const parser = new DOMParser(),
    DOM = parser.parseFromString(arg, "text/html");
  return DOM.body.childNodes[0] as HTMLElement;
};

const slideUp = (
  el: HTMLElement,
  duration = 300,
  callback = (el: HTMLElement) => {}
) => {
  el.style.transitionProperty = "height, margin, padding";
  el.style.transitionDuration = duration + "ms";
  el.style.height = el.offsetHeight + "px";
  el.offsetHeight;
  el.style.overflow = "hidden";
  el.style.height = "0";
  el.style.paddingTop = "0";
  el.style.paddingBottom = "0";
  el.style.marginTop = "0";
  el.style.marginBottom = "0";
  window.setTimeout(() => {
    el.style.display = "none";
    el.style.removeProperty("height");
    el.style.removeProperty("padding-top");
    el.style.removeProperty("padding-bottom");
    el.style.removeProperty("margin-top");
    el.style.removeProperty("margin-bottom");
    el.style.removeProperty("overflow");
    el.style.removeProperty("transition-duration");
    el.style.removeProperty("transition-property");
    callback(el);
  }, duration);
};

const slideDown = (
  el: HTMLElement,
  duration = 300,
  callback = (el: HTMLElement) => {}
) => {
  el.style.removeProperty("display");
  let display = window.getComputedStyle(el).display;
  if (display === "none") display = "block";
  el.style.display = display;
  const height = el.offsetHeight;
  el.style.overflow = "hidden";
  el.style.height = "0";
  el.style.paddingTop = "0";
  el.style.paddingBottom = "0";
  el.style.marginTop = "0";
  el.style.marginBottom = "0";
  el.offsetHeight;
  el.style.transitionProperty = "height, margin, padding";
  el.style.transitionDuration = duration + "ms";
  el.style.height = height + "px";
  el.style.removeProperty("padding-top");
  el.style.removeProperty("padding-bottom");
  el.style.removeProperty("margin-top");
  el.style.removeProperty("margin-bottom");
  window.setTimeout(() => {
    el.style.removeProperty("height");
    el.style.removeProperty("overflow");
    el.style.removeProperty("transition-duration");
    el.style.removeProperty("transition-property");
    callback(el);
  }, duration);
};

const concatErrors = (
  frontErrs: Array<ErrorObject> = [],
  backErrs: Array<string> = []
): Array<string> => {
  const pureFrontErrs: Array<string> = frontErrs.map((el) => el.$message) as Array<string>;
  return pureFrontErrs.concat(backErrs);
};

const routerGoBack = async (route?: string) => {
  const target = (route && !window.history.state.back) ? route : window.history.state.back;
  const { default: router } = await import("@/router");
  router.push(target);
};

const getCurrentToken = () => {
  const match = document.cookie.match(new RegExp('(^| )XSRF-TOKEN=([^;]+)'));
  return match ? decodeURIComponent(match[2]) : "";
}

function clearSmartLinksText(text: string) {
  const $html = document.createElement('div')
  $html.innerHTML = String(text)
  const $smartLinks = Array.from($html.querySelectorAll('[data-link="smart"]'))

  for (const $smartLink of $smartLinks) {
    $smartLink.innerHTML = ""
  }

  return $html.innerHTML
}

const phpRound = (num: number, precision = 0) => {
  return (num >= 0 ? 1 : -1) * Math.round(Math.abs(num) * Math.pow(10, precision)) / Math.pow(10, precision);
};

const declensionByNum = (value: number, words: string[]) => {
  value = Math.abs(value) % 100;
  const num = value % 10;
  if(value > 10 && value < 20) return words[2]??words[0];
  if(num > 1 && num < 5) return words[1]??words[0];
  if(num == 1) return words[0];
  return words[2]??words[0];
}

function setCookie(name: string, value: string | number | boolean, path = '/', options = {} as any) {
  options = {
    path: path,
    ...options
  };

  if (options.expires instanceof Date) {
    options.expires = options.expires.toUTCString();
  }

  let updatedCookie = encodeURIComponent(name) + "=" + encodeURIComponent(value);

  for (const optionKey in options) {
    updatedCookie += "; " + optionKey;
    const optionValue = options[optionKey];
    if (optionValue !== true) {
      updatedCookie += "=" + optionValue;
    }
  }

  document.cookie = updatedCookie;
}

function getCookie(name: string) {
  const matches = document.cookie.match(new RegExp(
    "(?:^|; )" + name.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, '\\$1') + "=([^;]*)"
  ));
  return matches ? decodeURIComponent(matches[1]) : undefined;
}

function deleteCookie(name: string, path = '/') {
  setCookie(name, "", path, {
    'max-age': -1
  })
}

const isBool = (val: string) => {
  return val === 'false' || val === 'undefined' || val === 'null' || val === '0' ? false : !!val
}

function isOverflow(el: HTMLElement) {
  return el.scrollHeight > el.clientHeight || el.scrollWidth > el.clientWidth;
}

const getQueryStringByParams = (params: any) => {
  return Object
    .keys(params)
    .filter(value => params[value]?.length != 0)
    .map(value => {
      let str = ''
      if (params[value] instanceof Array){
        params[value].forEach(function(item: string) {
          if (str) str += '&'
          str += `${value}=${encodeURIComponent(item)}`
        })
      } else {
        str = `${value}=${encodeURIComponent(params[value])}`
      }
      return str
    })
    .join('&');
}

const getDatetimeStart = (last: "day" | "2days" | "week", query: any) => {
  const dateStr = getPeriodStr(last).split(" - ")[0];
  if (query.date_begin) {
    return extractDateTime(query.date_begin+'');
  } else {
    const [day, month, year] = dateStr.split(".");
    const isoDateString = `${year}-${month}-${day}`;
    return new Date(isoDateString).setHours(0,0,0,0)
  }
}

const getDatetimeEnd = (last: "day" | "2days" | "week", query: any) => {
  const dateStr = getPeriodStr(last).split(" - ")[1];
  if (query.date_end) {
    return extractDateTime(query.date_end+'');
  } else {
    const [day, month, year] = dateStr.split(".");
    const isoDateString = `${year}-${month}-${day}`;
    return new Date(isoDateString).setHours(23,59,59,999)
  }
}

const extractDateTime = (dateStr: string):Date => {
  const [date, time] = (dateStr).split(' ');
  const [day, month, year] = date.split(/[. :]/);
  const formattedDateString = `${month}/${day}/${year} ${time}`;
  return new Date(formattedDateString);
}

const getPeriodStr = (last: "day" | "2days" | "week"): string => {
  const currentDate = new Date();
  const dateEndString = ("0" + currentDate.getDate()).slice(-2) + "." + ("0"+(currentDate.getMonth() + 1)).slice(-2) + "." + currentDate.getFullYear();

  if (last == "day") {
    return `${dateEndString} - ${dateEndString}`;
  }

  if (last == "2days") {
    const prevDate = new Date(currentDate);
    prevDate.setDate(currentDate.getDate() - 1)
    const dateBeginString = ("0" + prevDate.getDate()).slice(-2) + "." + ("0"+(prevDate.getMonth() + 1)).slice(-2) + "." + prevDate.getFullYear();

    return `${dateBeginString} - ${dateEndString}`;
  }

  if (last == "week") {
    const prevDate = new Date(currentDate);
    prevDate.setDate(currentDate.getDate() - 7)
    const dateBeginString = ("0" + prevDate.getDate()).slice(-2) + "." + ("0"+(prevDate.getMonth() + 1)).slice(-2) + "." + prevDate.getFullYear();

    return `${dateBeginString} - ${dateEndString}`;
  }

  return "";
};

const highlight = (needle: string, text: string) => {
  if (needle === null) {
    return
  }

  const index = text.search(new RegExp(needle, 'i'))

  if (index >= 0) {
    text = text.substring(0, index)
      + '<mark>'
      + text.substring(index, index + needle.length)
      + '</mark>'
      + text.substring(index + needle.length, text.length)
  }


  return text
}

const getRandomString = (): string => Math.random().toString(36).slice(2, 7);

const makeLinks = (text: string): string => {
  if (!text) return "";

  const linkRegexp = /\[(.*?)\]\((http(s)?:\/\/(www\.)?(((localhost)?(:\d+)?)|([-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}))\b([-a-zA-Z0-9()@:%_\+.~#?&//=]*))\)/g;

  return text.replace(linkRegexp, "<a class=\"text-blue-600 dark:text-blue-500 italic font-bold\" href=\"$2\">$1</a>");
};

const isImage = (file: IFile) => {
  const [, fileExtension] = file.name.split('.')

  const extensions = new Set([
    '3dv',
    'ai',
    'amf',
    'art',
    'ase',
    'awg',
    'blp',
    'bmp',
    'bw',
    'cd5',
    'cdr',
    'cgm',
    'cit',
    'cmx',
    'cpt',
    'cr2',
    'cur',
    'cut',
    'dds',
    'dib',
    'djvu',
    'dxf',
    'e2d',
    'ecw',
    'egt',
    'emf',
    'eps',
    'exif',
    'fs',
    'gbr',
    'gif',
    'gpl',
    'grf',
    'hdp',
    'heic',
    'heif',
    'icns',
    'ico',
    'iff',
    'int',
    'inta',
    'jfif',
    'jng',
    'jp2',
    'jpeg',
    'jpg',
    'jps',
    'jxr',
    'lbm',
    'liff',
    'max',
    'miff',
    'mng',
    'msp',
    'nef',
    'nitf',
    'nrrd',
    'odg',
    'ota',
    'pam',
    'pbm',
    'pc1',
    'pc2',
    'pc3',
    'pcf',
    'pct',
    'pcx',
    'pdd',
    'pdn',
    'pgf',
    'pgm',
    'PI1',
    'PI2',
    'PI3',
    'pict',
    'png',
    'pnm',
    'pns',
    'ppm',
    'psb',
    'psd',
    'psp',
    'px',
    'pxm',
    'pxr',
    'qfx',
    'ras',
    'raw',
    'rgb',
    'rgba',
    'rle',
    'sct',
    'sgi',
    'sid',
    'stl',
    'sun',
    'svg',
    'sxd',
    'tga',
    'tif',
    'tiff',
    'v2d',
    'vnd',
    'vrml',
    'vtf',
    'wdp',
    'webp',
    'wmf',
    'x3d',
    'xar',
    'xbm',
    'xcf',
    'xpm',
  ]);

  return extensions.has(fileExtension);
}

export {
  dayjs,
  cutText,
  formatDate,
  capitalizeFirstLetter,
  onlyNumber,
  formatCurrency,
  timeAgo,
  diffTimeByNow,
  isset,
  toRaw,
  randomNumbers,
  toRGB,
  stringToHTML,
  metaEnv,
  slideUp,
  slideDown,
  routerGoBack,
  concatErrors,
  getCurrentToken,
  clearSmartLinksText,
  phpRound,
  declensionByNum,
  setCookie,
  getCookie,
  deleteCookie,
  isBool,
  isOverflow,
  getQueryStringByParams,
  getDatetimeStart,
  getDatetimeEnd,
  extractDateTime,
  getPeriodStr,
  highlight,
  getRandomString,
  makeLinks,
  isImage,
};
